@@ -82,6 +82,9 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "before", SLEN("before") },
 	{ "after", SLEN("after") },
 
+	{ "align-content", SLEN("align-content") },
+	{ "align-items", SLEN("align-items") },
+	{ "align-self", SLEN("align-self") },
 	{ "azimuth", SLEN("azimuth") },
 	{ "background", SLEN("background") },
 	{ "background-attachment", SLEN("background-attachment") },
@@ -112,6 +115,7 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "border-top-width", SLEN("border-top-width") },
 	{ "border-width", SLEN("border-width") },
 	{ "bottom", SLEN("bottom") },
+	{ "box-sizing", SLEN("box-sizing") },
 	{ "break-after", SLEN("break-after") },
 	{ "break-before", SLEN("break-before") },
 	{ "break-inside", SLEN("break-inside") },
@@ -140,6 +144,13 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "display", SLEN("display") },
 	{ "elevation", SLEN("elevation") },
 	{ "empty-cells", SLEN("empty-cells") },
+	{ "flex", SLEN("flex") },
+	{ "flex-basis", SLEN("flex-basis") },
+	{ "flex-direction", SLEN("flex-direction") },
+	{ "flex-flow", SLEN("flex-flow") },
+	{ "flex-grow", SLEN("flex-grow") },
+	{ "flex-shrink", SLEN("flex-shrink") },
+	{ "flex-wrap", SLEN("flex-wrap") },
 	{ "float", SLEN("float") },
 	{ "font", SLEN("font") },
 	{ "font-family", SLEN("font-family") },
@@ -148,6 +159,7 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "font-variant", SLEN("font-variant") },
 	{ "font-weight", SLEN("font-weight") },
 	{ "height", SLEN("height") },
+	{ "justify-content", SLEN("justify-content") },
 	{ "left", SLEN("left") },
 	{ "letter-spacing", SLEN("letter-spacing") },
 	{ "line-height", SLEN("line-height") },
@@ -165,6 +177,7 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "min-height", SLEN("min-height") },
 	{ "min-width", SLEN("min-width") },
 	{ "opacity", SLEN("opacity") },
+	{ "order", SLEN("order") },
 	{ "orphans", SLEN("orphans") },
 	{ "outline", SLEN("outline") },
 	{ "outline-color", SLEN("outline-color") },
@@ -412,6 +425,24 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "horizontal-tb", SLEN("horizontal-tb") },
 	{ "vertical-rl", SLEN("vertical-rl") },
 	{ "vertical-lr", SLEN("vertical-lr") },
+	{ "content-box", SLEN("content-box") },
+	{ "border-box", SLEN("border-box") },
+	{ "stretch", SLEN("stretch") },
+	{ "inline-flex", SLEN("inline-flex") },
+	{ "flex-start", SLEN("flex-start") },
+	{ "flex-end", SLEN("flex-end") },
+	{ "space-between", SLEN("space-between") },
+	{ "space-around", SLEN("space-around") },
+	{ "space-evenly", SLEN("space-evenly") },
+	{ "row", SLEN("row") },
+	{ "row-reverse", SLEN("row-reverse") },
+	{ "column-reverse", SLEN("column-reverse") },
+	{ "wrap", SLEN("wrap") },
+	{ "wrap-reverse", SLEN("wrap-reverse") },
+	{ "and", SLEN("and") },
+	{ "or", SLEN("or") },
+	{ "only", SLEN("only") },
+	{ "infinite", SLEN("infinite") },
 
 	{ "aliceblue", SLEN("aliceblue") },
 	{ "antiquewhite", SLEN("antiquewhite") },
@@ -618,5 +649,3 @@ void css__propstrings_unref(void)
 			lwc_string_unref(css__propstrings.strings[i]);
 	}
 }
-
-
