@@ -10,6 +10,7 @@
 
 #include "stylesheet.h"
 #include "select/hash.h"
+#include "select/mq.h"
 #include "utils/utils.h"
 
 #undef PRINT_CHAIN_BLOOM_DETAILS
@@ -43,7 +44,7 @@ static hash_entry empty_slot;
 
 static inline lwc_string *_class_name(const css_selector *selector);
 static inline lwc_string *_id_name(const css_selector *selector);
-static css_error _insert_into_chain(css_selector_hash *ctx, hash_entry *head, 
+static css_error _insert_into_chain(css_selector_hash *ctx, hash_entry *head,
 		const css_selector *selector);
 static css_error _remove_from_chain(css_selector_hash *ctx, hash_entry *head,
 		const css_selector *selector);
@@ -106,36 +107,6 @@ static inline bool _chain_good_for_element_name(const css_selector *selector,
 	return true;
 }
 
-/**
- * Test whether the rule applies for current media.
- *
- * \param rule		Rule to test
- * \meaid media		Current media type(s)
- * \return true iff chain's rule applies for media
- */
-static inline bool _rule_good_for_media(const css_rule *rule, uint64_t media)
-{
-	bool applies = true;
-	const css_rule *ancestor = rule;
-
-	while (ancestor != NULL) {
-		const css_rule_media *m = (const css_rule_media *) ancestor;
-
-		if (ancestor->type == CSS_RULE_MEDIA &&
-				(m->media & media) == 0) {
-			applies = false;
-			break;
-		}
-
-		if (ancestor->ptype != CSS_RULE_PARENT_STYLESHEET)
-			ancestor = ancestor->parent;
-		else
-			ancestor = NULL;
-	}
-
-	return applies;
-}
-
 
 /**
  * Create a hash
@@ -183,7 +154,7 @@ css_error css__selector_hash_create(css_selector_hash **hash)
 
 	/* Universal chain head already initiliased by calloc of `h`. */
 
-	h->hash_size = sizeof(css_selector_hash) + 
+	h->hash_size = sizeof(css_selector_hash) +
 			DEFAULT_SLOTS * sizeof(hash_entry) +
 			DEFAULT_SLOTS * sizeof(hash_entry) +
 			DEFAULT_SLOTS * sizeof(hash_entry);
@@ -396,7 +367,7 @@ css_error css__selector_hash_find(css_selector_hash *hash,
 				if (css_bloom_in_bloom(
 						head->sel_chain_bloom,
 						req->node_bloom) &&
-				    _rule_good_for_media(head->sel->rule,
+				    mq_rule_good_for_media(head->sel->rule,
 						req->media)) {
 					/* Found a match */
 					break;
@@ -474,7 +445,7 @@ css_error css__selector_hash_find_by_class(css_selector_hash *hash,
 							head->sel,
 							&(req->qname),
 							req->uni) &&
-					    _rule_good_for_media(
+					    mq_rule_good_for_media(
 							head->sel->rule,
 							req->media)) {
 						/* Found a match */
@@ -554,7 +525,7 @@ css_error css__selector_hash_find_by_id(css_selector_hash *hash,
 							head->sel,
 							&req->qname,
 							req->uni) &&
-					    _rule_good_for_media(
+					    mq_rule_good_for_media(
 							head->sel->rule,
 							req->media)) {
 						/* Found a match */
@@ -605,7 +576,7 @@ css_error css__selector_hash_find_universal(css_selector_hash *hash,
 			    css_bloom_in_bloom(
 					head->sel_chain_bloom,
 					req->node_bloom) &&
-			    _rule_good_for_media(head->sel->rule,
+			    mq_rule_good_for_media(head->sel->rule,
 					req->media)) {
 				/* Found a match */
 				break;
@@ -810,7 +781,7 @@ static void print_chain_bloom_details(css_bloom bloom[CSS_BLOOM_SIZE])
  * \return CSS_OK    on success,
  *         CSS_NOMEM on memory exhaustion.
  */
-css_error _insert_into_chain(css_selector_hash *ctx, hash_entry *head, 
+css_error _insert_into_chain(css_selector_hash *ctx, hash_entry *head,
 		const css_selector *selector)
 {
 	if (head->sel == NULL) {
@@ -836,7 +807,7 @@ css_error _insert_into_chain(css_selector_hash *ctx, hash_entry *head,
 
 			/* Sort by ascending rule index */
 			if (search->sel->specificity == selector->specificity &&
-					search->sel->rule->index > 
+					search->sel->rule->index >
 					selector->rule->index)
 				break;
 
@@ -844,6 +815,16 @@ css_error _insert_into_chain(css_selector_hash *ctx, hash_entry *head,
 			search = search->next;
 		} while (search != NULL);
 
+		if (prev == NULL) {
+			*entry = *head;
+			head->next = entry;
+
+			entry = head;
+		} else {
+			entry->next = prev->next;
+			prev->next = entry;
+		}
+
 		entry->sel = selector;
 		_chain_bloom_generate(selector, entry->sel_chain_bloom);
 
@@ -851,17 +832,6 @@ css_error _insert_into_chain(css_selector_hash *ctx, hash_entry *head,
 		print_chain_bloom_details(entry->sel_chain_bloom);
 #endif
 
-		if (prev == NULL) {
-			hash_entry temp;
-			entry->next = entry;
-			temp = *entry;
-			*entry = *head;
-			*head = temp;
-		} else {
-			entry->next = prev->next;
-			prev->next = entry;
-		}
-
 		ctx->hash_size += sizeof(hash_entry);
 	}
 
@@ -949,7 +919,7 @@ css_error _iterate_elements(
 				if (css_bloom_in_bloom(
 						head->sel_chain_bloom,
 						req->node_bloom) &&
-				    _rule_good_for_media(head->sel->rule,
+				    mq_rule_good_for_media(head->sel->rule,
 						req->media)) {
 					/* Found a match */
 					break;
@@ -1008,7 +978,7 @@ css_error _iterate_classes(
 							head->sel,
 							&(req->qname),
 							req->uni) &&
-					    _rule_good_for_media(
+					    mq_rule_good_for_media(
 							head->sel->rule,
 							req->media)) {
 						/* Found a match */
@@ -1069,7 +1039,7 @@ css_error _iterate_ids(
 							head->sel,
 							&req->qname,
 							req->uni) &&
-					    _rule_good_for_media(
+					    mq_rule_good_for_media(
 							head->sel->rule,
 							req->media)) {
 						/* Found a match */
@@ -1113,7 +1083,7 @@ css_error _iterate_universal(
 			    css_bloom_in_bloom(
 					head->sel_chain_bloom,
 					req->node_bloom) &&
-			    _rule_good_for_media(head->sel->rule,
+			    mq_rule_good_for_media(head->sel->rule,
 					req->media)) {
 				/* Found a match */
 				break;
