@@ -56,6 +56,8 @@ dom_exception _dom_event_initialise(dom_event *evt)
 
 	evt->refcnt = 1;
 	evt->in_dispatch = false;
+	evt->is_initialised = false;
+	evt->is_trusted = true;
 
 	return DOM_NO_ERR;
 }
@@ -252,6 +254,7 @@ dom_exception _dom_event_init(dom_event *evt, dom_string *type,
 	evt->type = dom_string_ref(type);
 	evt->bubble = bubble;
 	evt->cancelable = cancelable;
+	evt->is_initialised = true;
 
 	evt->timestamp = time(NULL);
 
@@ -333,7 +336,63 @@ dom_exception _dom_event_init_ns(dom_event *evt, dom_string *namespace,
 
 	evt->bubble = bubble;
 	evt->cancelable = cancelable;
+	evt->is_initialised = true;
 
 	return DOM_NO_ERR;
 }
 
+/**
+ * Return whether or not the event is in dispatch currently.
+ *
+ * \param evt The event object
+ * \param result Pointer to result object
+ * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
+ */
+dom_exception _dom_event_in_dispatch(dom_event *evt, bool *result)
+{
+	*result = evt->in_dispatch;
+
+	return DOM_NO_ERR;
+}
+
+/**
+ * Return whether or not the event is currently initialised.
+ *
+ * \param evt The event object
+ * \param result Pointer to result object
+ * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
+ */
+dom_exception _dom_event_is_initialised(dom_event *evt, bool *result)
+{
+	*result = evt->is_initialised;
+
+	return DOM_NO_ERR;
+}
+
+/**
+ * Return whether or not the event is currently trusted.
+ *
+ * \param evt The event object
+ * \param result Pointer to result object
+ * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
+ */
+dom_exception _dom_event_get_is_trusted(dom_event *evt, bool *result)
+{
+	*result = evt->is_trusted;
+
+	return DOM_NO_ERR;
+}
+
+/**
+ * Set whether or not the event is currently trusted.
+ *
+ * \param evt The event object
+ * \param trusted Whether or not the event is to be trusted
+ * \return DOM_NO_ERR on success, appropriate dom_exception on failure.
+ */
+dom_exception _dom_event_set_is_trusted(dom_event *evt, bool trusted)
+{
+	evt->is_trusted = trusted;
+
+	return DOM_NO_ERR;
+}
