@@ -215,7 +215,8 @@ dom_exception dom_html_collection_named_item(dom_html_collection *col,
 	struct dom_node_internal *n = col->root;
 	dom_html_document *doc = (dom_html_document *)dom_node_get_owner(n);
 	dom_exception err;
-        while (n != NULL) {
+
+	while (n != NULL) {
 		if (n->type == DOM_ELEMENT_NODE &&
 		    col->ic(n, col->ctx) == true) {
 			dom_string *id = NULL;
@@ -227,31 +228,32 @@ dom_exception dom_html_collection_named_item(dom_html_collection *col,
 				return err;
 			}
 
-			if (id != NULL && dom_string_isequal(name, id)) {
-				*node = (struct dom_node *) n;
-				dom_node_ref(n);
-				dom_string_unref(id);
+			if (id != NULL) {
+				if (dom_string_isequal(name, id)) {
+					*node = dom_node_ref(n);
+					dom_string_unref(id);
 
-				return DOM_NO_ERR;
+					return DOM_NO_ERR;
+				}
+				dom_string_unref(id);
 			}
 
-			if (id != NULL)
-				dom_string_unref(id);
-
 			/* Check for Name attr if id not matched/found */
 			err = _dom_element_get_attribute((dom_element *)n,
 					doc->memoised[hds_name], &id_name);
 			if(err != DOM_NO_ERR) {
 				return err;
 			}
-			if (id_name != NULL && dom_string_isequal(name, id_name)) {
-				*node = (struct dom_node *) n;
-				dom_node_ref(n);
+
+			if (id_name != NULL) {
+				if (dom_string_isequal(name, id_name)) {
+					*node = dom_node_ref(n);
+					dom_string_unref(id_name);
+
+					return DOM_NO_ERR;
+				}
 				dom_string_unref(id_name);
-
-				return DOM_NO_ERR;
 			}
-
 		}
 
 		/* Depth first iterating */
