@@ -22,7 +22,7 @@
 #endif
 
 #ifndef UNUSED
-#define UNUSED(x) ((x)=(x))
+#define UNUSED(x) ((void)(x))
 #endif
 
 #endif
