@@ -25,58 +25,116 @@ static const struct {
 	size_t len;
 	element_type type;
 } name_type_map[] = {
-	{ S("address"), ADDRESS },	{ S("area"), AREA },
-	{ S("base"), BASE },		{ S("basefont"), BASEFONT },
-	{ S("bgsound"), BGSOUND },	{ S("blockquote"), BLOCKQUOTE },
-	{ S("body"), BODY },		{ S("br"), BR },
-	{ S("center"), CENTER },	{ S("col"), COL },
-	{ S("colgroup"), COLGROUP },	{ S("dd"), DD },
-	{ S("dir"), DIR },		{ S("div"), DIV },
-	{ S("dl"), DL },		{ S("dt"), DT },
-	{ S("embed"), EMBED },		{ S("fieldset"), FIELDSET },
-	{ S("form"), FORM },		{ S("frame"), FRAME },
-	{ S("frameset"), FRAMESET },	{ S("h1"), H1 },
-	{ S("h2"), H2 },		{ S("h3"), H3 },
-	{ S("h4"), H4 },		{ S("h5"), H5 },
-	{ S("h6"), H6 },		{ S("head"), HEAD },
-	{ S("hr"), HR },		{ S("iframe"), IFRAME },
-	{ S("image"), IMAGE },		{ S("img"), IMG },
-	{ S("input"), INPUT },		{ S("isindex"), ISINDEX },
-	{ S("li"), LI },		{ S("link"), LINK },
+	{ S("address"), ADDRESS },
+	{ S("area"), AREA },
+	{ S("article"), ARTICLE },
+	{ S("aside"), ASIDE },
+	{ S("base"), BASE },
+	{ S("basefont"), BASEFONT },
+	{ S("bgsound"), BGSOUND },
+	{ S("blockquote"), BLOCKQUOTE },
+	{ S("body"), BODY },
+	{ S("br"), BR },
+	{ S("center"), CENTER },
+	{ S("col"), COL },
+	{ S("colgroup"), COLGROUP },
+	{ S("command"), COMMAND },
+	{ S("dd"), DD },
+	{ S("details"), DETAILS },
+	{ S("dialog"), DIALOG },
+	{ S("dir"), DIR },
+	{ S("div"), DIV },
+	{ S("dl"), DL },
+	{ S("dt"), DT },
+	{ S("embed"), EMBED },
+	{ S("fieldset"), FIELDSET },
+	{ S("figcaption"), FIGCAPTION },
+	{ S("figure"), FIGURE },
+	{ S("footer"), FOOTER },
+	{ S("form"), FORM },
+	{ S("frame"), FRAME },
+	{ S("frameset"), FRAMESET },
+	{ S("h1"), H1 },
+	{ S("h2"), H2 },
+	{ S("h3"), H3 },
+	{ S("h4"), H4 },
+	{ S("h5"), H5 },
+	{ S("h6"), H6 },
+	{ S("head"), HEAD },
+	{ S("hr"), HR },
+	{ S("iframe"), IFRAME },
+	{ S("image"), IMAGE },
+	{ S("img"), IMG },
+	{ S("input"), INPUT },
+	{ S("isindex"), ISINDEX },
+	{ S("li"), LI },
+	{ S("link"), LINK },
 	{ S("listing"), LISTING },
 	{ S("menu"), MENU },
-	{ S("meta"), META },		{ S("noembed"), NOEMBED },
-	{ S("noframes"), NOFRAMES },	{ S("noscript"), NOSCRIPT },
-	{ S("ol"), OL },		{ S("optgroup"), OPTGROUP },
-	{ S("option"), OPTION },	{ S("output"), OUTPUT },
-	{ S("p"), P },			{ S("param"), PARAM },
-	{ S("plaintext"), PLAINTEXT },	{ S("pre"), PRE },
-	{ S("script"), SCRIPT },	{ S("select"), SELECT },
-	{ S("spacer"), SPACER },	{ S("style"), STYLE },
- 	{ S("tbody"), TBODY },		{ S("textarea"), TEXTAREA },
-	{ S("tfoot"), TFOOT },		{ S("thead"), THEAD },
-	{ S("title"), TITLE },		{ S("tr"), TR },
-	{ S("ul"), UL },		{ S("wbr"), WBR },
-	{ S("applet"), APPLET },	{ S("button"), BUTTON },
-	{ S("caption"), CAPTION },	{ S("html"), HTML },
-	{ S("marquee"), MARQUEE },	{ S("object"), OBJECT },
-	{ S("table"), TABLE },		{ S("td"), TD },
+	{ S("meta"), META },
+	{ S("noembed"), NOEMBED },
+	{ S("noframes"), NOFRAMES },
+	{ S("noscript"), NOSCRIPT },
+	{ S("ol"), OL },
+	{ S("optgroup"), OPTGROUP },
+	{ S("option"), OPTION },
+	{ S("output"), OUTPUT },
+	{ S("p"), P },
+	{ S("param"), PARAM },
+	{ S("plaintext"), PLAINTEXT },
+	{ S("pre"), PRE },
+	{ S("script"), SCRIPT },
+	{ S("select"), SELECT },
+	{ S("spacer"), SPACER },
+	{ S("style"), STYLE },
+	{ S("summary"), SUMMARY },
+	{ S("tbody"), TBODY },
+	{ S("textarea"), TEXTAREA },
+	{ S("tfoot"), TFOOT },
+	{ S("thead"), THEAD },
+	{ S("title"), TITLE },
+	{ S("tr"), TR },
+	{ S("ul"), UL },
+	{ S("wbr"), WBR },
+
+	{ S("applet"), APPLET },
+	{ S("button"), BUTTON },
+	{ S("caption"), CAPTION },
+	{ S("html"), HTML },
+	{ S("marquee"), MARQUEE },
+	{ S("object"), OBJECT },
+	{ S("table"), TABLE },
+	{ S("td"), TD },
 	{ S("th"), TH },
-	{ S("a"), A },			{ S("b"), B },
-	{ S("big"), BIG },		{ S("em"), EM },
-	{ S("font"), FONT },		{ S("i"), I },
-	{ S("nobr"), NOBR },		{ S("s"), S },
-	{ S("small"), SMALL },		{ S("strike"), STRIKE },
-	{ S("strong"), STRONG },	{ S("tt"), TT },
-	{ S("u"), U },			{ S("xmp"), XMP },
 
-	{ S("math"), MATH },		{ S("mglyph"), MGLYPH },
+	{ S("a"), A },
+	{ S("b"), B },
+	{ S("big"), BIG },
+	{ S("em"), EM },
+	{ S("font"), FONT },
+	{ S("i"), I },
+	{ S("nobr"), NOBR },
+	{ S("s"), S },
+	{ S("small"), SMALL },
+	{ S("strike"), STRIKE },
+	{ S("strong"), STRONG },
+	{ S("tt"), TT },
+	{ S("u"), U },
+
+	{ S("xmp"), XMP },
+
+	{ S("math"), MATH },
+	{ S("mglyph"), MGLYPH },
 	{ S("malignmark"), MALIGNMARK },
-	{ S("mi"), MI },		{ S("mo"), MO },
-	{ S("mn"), MN },		{ S("ms"), MS },
-	{ S("mtext"), MTEXT },		{ S("annotation-xml"), ANNOTATION_XML },
+	{ S("mi"), MI },
+	{ S("mo"), MO },
+	{ S("mn"), MN },
+	{ S("ms"), MS },
+	{ S("mtext"), MTEXT },
+	{ S("annotation-xml"), ANNOTATION_XML },
 
-	{ S("svg"), SVG },		{ S("desc"), DESC },
+	{ S("svg"), SVG },
+	{ S("desc"), DESC },
 	{ S("foreignobject"), FOREIGNOBJECT },
 };
 
@@ -1391,15 +1449,21 @@ hubbub_error formatting_list_remove(hubbub_treebuilder *treebuilder,
 	*node = entry->details.node;
 	*stack_index = entry->stack_index;
 
-	if (entry->prev == NULL)
+	if (entry->prev == NULL) {
+		assert(treebuilder->context.formatting_list == entry);
 		treebuilder->context.formatting_list = entry->next;
-	else
+	} else {
+		assert(treebuilder->context.formatting_list != entry);
 		entry->prev->next = entry->next;
+	}
 
-	if (entry->next == NULL)
+	if (entry->next == NULL) {
+		assert(treebuilder->context.formatting_list_end == entry);
 		treebuilder->context.formatting_list_end = entry->prev;
-	else
+	} else {
+		assert(treebuilder->context.formatting_list_end != entry);
 		entry->next->prev = entry->prev;
+	}
 
 	free(entry);
 
