@@ -26,8 +26,8 @@
  * around the fetcher specific methods.
  *
  * Active fetches are held in the circular linked list ::fetch_ring. There may
- * be at most ::option_max_fetchers_per_host active requests per Host: header.
- * There may be at most ::option_max_fetchers active requests overall. Inactive
+ * be at most nsoption max_fetchers_per_host active requests per Host: header.
+ * There may be at most nsoption max_fetchers active requests overall. Inactive
  * fetches are stored in the ::queue_ring waiting for use.
  */
 
@@ -46,7 +46,6 @@
 #include "utils/log.h"
 #include "utils/messages.h"
 #include "utils/nsurl.h"
-#include "utils/utils.h"
 #include "utils/ring.h"
 #include "netsurf/misc.h"
 #include "desktop/gui_internal.h"
@@ -61,16 +60,6 @@
 #include "javascript/fetcher.h"
 #include "content/urldb.h"
 
-/* Define this to turn on verbose fetch logging */
-#undef DEBUG_FETCH_VERBOSE
-
-/** Verbose fetcher logging */
-#ifdef DEBUG_FETCH_VERBOSE
-#define FETCH_LOG(x...) LOG(x...)
-#else
-#define FETCH_LOG(x...)
-#endif
-
 /** The maximum number of fetchers that can be added */
 #define MAX_FETCHERS 10
 
@@ -109,6 +98,7 @@ struct fetch {
 	int fetcherd;           /**< Fetcher descriptor for this fetch */
 	void *fetcher_handle;	/**< The handle for the fetcher. */
 	bool fetch_is_active;	/**< This fetch is active. */
+	fetch_msg_type last_msg;/**< The last message sent for this fetch */
 	struct fetch *r_prev;	/**< Previous active fetch in ::fetch_ring. */
 	struct fetch *r_next;	/**< Next active fetch in ::fetch_ring. */
 };
@@ -159,8 +149,10 @@ static int get_fetcher_for_scheme(lwc_string *scheme)
 static bool fetch_dispatch_job(struct fetch *fetch)
 {
 	RING_REMOVE(queue_ring, fetch);
-	FETCH_LOG("Attempting to start fetch %p, fetcher %p, url %s", fetch,
-	     fetch->fetcher_handle, nsurl_access(fetch->url));
+	NSLOG(fetch, DEBUG,
+	      "Attempting to start fetch %p, fetcher %p, url %s", fetch,
+	      fetch->fetcher_handle,
+	      nsurl_access(fetch->url));
 
 	if (!fetchers[fetch->fetcherd].ops.start(fetch->fetcher_handle)) {
 		RING_INSERT(queue_ring, fetch); /* Put it back on the end of the queue */
@@ -211,25 +203,25 @@ static bool fetch_choose_and_dispatch(void)
 
 static void dump_rings(void)
 {
-#ifdef DEBUG_FETCH_VERBOSE
 	struct fetch *q;
 	struct fetch *f;
 
 	q = queue_ring;
 	if (q) {
 		do {
-			LOG("queue_ring: %s", nsurl_access(q->url));
+			NSLOG(fetch, DEBUG, "queue_ring: %s",
+			      nsurl_access(q->url));
 			q = q->r_next;
 		} while (q != queue_ring);
 	}
 	f = fetch_ring;
 	if (f) {
 		do {
-			LOG("fetch_ring: %s", nsurl_access(f->url));
+			NSLOG(fetch, DEBUG, "fetch_ring: %s",
+			      nsurl_access(f->url));
 			f = f->r_next;
 		} while (f != fetch_ring);
 	}
-#endif
 }
 
 /**
@@ -245,7 +237,10 @@ static bool fetch_dispatch_jobs(void)
 	RING_GETSIZE(struct fetch, queue_ring, all_queued);
 	RING_GETSIZE(struct fetch, fetch_ring, all_active);
 
-	FETCH_LOG("queue_ring %i, fetch_ring %i", all_queued, all_active);
+	NSLOG(fetch, DEBUG,
+	      "queue_ring %i, fetch_ring %i",
+	      all_queued,
+	      all_active);
 	dump_rings();
 
 	while ((all_queued != 0) &&
@@ -253,12 +248,14 @@ static bool fetch_dispatch_jobs(void)
 	       fetch_choose_and_dispatch()) {
 			all_queued--;
 			all_active++;
-			FETCH_LOG("%d queued, %d fetching",
-				   all_queued, all_active);
+			NSLOG(fetch, DEBUG,
+			      "%d queued, %d fetching",
+			      all_queued,
+			      all_active);
 	}
 
-	FETCH_LOG("Fetch ring is now %d elements.", all_active);
-	FETCH_LOG("Queue ring is now %d elements.", all_queued);
+	NSLOG(fetch, DEBUG, "Fetch ring is now %d elements.", all_active);
+	NSLOG(fetch, DEBUG, "Queue ring is now %d elements.", all_queued);
 
 	return (all_active > 0);
 }
@@ -268,7 +265,7 @@ static void fetcher_poll(void *unused)
 	int fetcherd;
 
 	if (fetch_dispatch_jobs()) {
-		FETCH_LOG("Polling fetchers");
+		NSLOG(fetch, DEBUG, "Polling fetchers");
 		for (fetcherd = 0; fetcherd < MAX_FETCHERS; fetcherd++) {
 			if (fetchers[fetcherd].refcount > 0) {
 				/* fetcher present */
@@ -342,7 +339,10 @@ void fetcher_quit(void)
 			 * the reference count to allow the fetcher to
 			 * be stopped.
 			 */
-			LOG("Fetcher for scheme %s still has %d active users at quit.", lwc_string_data(fetchers[fetcherd].scheme), fetchers[fetcherd].refcount);
+			NSLOG(fetch, INFO,
+			      "Fetcher for scheme %s still has %d active users at quit.",
+			      lwc_string_data(fetchers[fetcherd].scheme),
+			      fetchers[fetcherd].refcount);
 
 			fetchers[fetcherd].refcount = 1;
 		}
@@ -392,12 +392,12 @@ fetch_fdset(fd_set *read_fd_set,
 	int fetcherd; /* fetcher index */
 
 	if (!fetch_dispatch_jobs()) {
-		FETCH_LOG("No jobs");
+		NSLOG(fetch, DEBUG, "No jobs");
 		*maxfd_out = -1;
 		return NSERROR_OK;
 	}
 
-	FETCH_LOG("Polling fetchers");
+	NSLOG(fetch, DEBUG, "Polling fetchers");
 
 	for (fetcherd = 0; fetcherd < MAX_FETCHERS; fetcherd++) {
 		if (fetchers[fetcherd].refcount > 0) {
@@ -463,7 +463,7 @@ fetch_start(nsurl *url,
 	lwc_string *scheme;
 	bool match;
 
-	fetch = malloc(sizeof (*fetch));
+	fetch = calloc(1, sizeof (*fetch));
 	if (fetch == NULL) {
 		return NSERROR_NOMEM;
 	}
@@ -480,20 +480,13 @@ fetch_start(nsurl *url,
 		return NSERROR_NO_FETCH_HANDLER;
 	}
 
-	FETCH_LOG("fetch %p, url '%s'", fetch, nsurl_access(url));
+	NSLOG(fetch, DEBUG, "fetch %p, url '%s'", fetch, nsurl_access(url));
 
 	/* construct a new fetch structure */
 	fetch->callback = callback;
 	fetch->url = nsurl_ref(url);
 	fetch->verifiable = verifiable;
 	fetch->p = p;
-	fetch->http_code = 0;
-	fetch->r_prev = NULL;
-	fetch->r_next = NULL;
-	fetch->referer = NULL;
-	fetch->send_referer = false;
-	fetch->fetcher_handle = NULL;
-	fetch->fetch_is_active = false;
 	fetch->host = nsurl_get_component(url, NSURL_HOST);
 
 	if (referer != NULL) {
@@ -572,7 +565,7 @@ fetch_start(nsurl *url,
 
 	/* Ask the queue to run. */
 	if (fetch_dispatch_jobs()) {
-		FETCH_LOG("scheduling poll");
+		NSLOG(fetch, DEBUG, "scheduling poll");
 		/* schedule active fetchers to run again in 10ms */
 		guit->misc->schedule(10, fetcher_poll, NULL);
 	}
@@ -585,7 +578,9 @@ fetch_start(nsurl *url,
 void fetch_abort(struct fetch *f)
 {
 	assert(f);
-	FETCH_LOG("fetch %p, fetcher %p, url '%s'", f, f->fetcher_handle,
+	f->last_msg = FETCH__INTERNAL_ABORTED;
+	NSLOG(fetch, DEBUG,
+	      "fetch %p, fetcher %p, url '%s'", f, f->fetcher_handle,
 	     nsurl_access(f->url));
 	fetchers[f->fetcherd].ops.abort(f->fetcher_handle);
 }
@@ -593,7 +588,24 @@ void fetch_abort(struct fetch *f)
 /* exported interface documented in content/fetch.h */
 void fetch_free(struct fetch *f)
 {
-	FETCH_LOG("Freeing fetch %p, fetcher %p", f, f->fetcher_handle);
+	if (f->last_msg < FETCH_MIN_FINISHED_MSG) {
+		/* We didn't finish, so tell our user that an error occurred */
+		fetch_msg msg;
+
+		msg.type = FETCH_ERROR;
+		msg.data.error = "FetchFailedToFinish";
+
+		NSLOG(fetch, CRITICAL,
+		      "During the fetch of %s, the fetcher did not finish.",
+		      nsurl_access(f->url));
+
+		fetch_send_callback(&msg, f);
+	}
+
+	NSLOG(fetch, DEBUG,
+	      "Freeing fetch %p, fetcher %p",
+	      f,
+	      f->fetcher_handle);
 
 	fetchers[f->fetcherd].ops.free(f->fetcher_handle);
 
@@ -643,13 +655,6 @@ long fetch_http_code(struct fetch *fetch)
 	return fetch->http_code;
 }
 
-/* exported interface documented in content/fetch.h */
-bool fetch_get_verifiable(struct fetch *fetch)
-{
-	assert(fetch);
-
-	return fetch->verifiable;
-}
 
 /* exported interface documented in content/fetch.h */
 struct fetch_multipart_data *
@@ -716,6 +721,23 @@ fetch_multipart_data_clone(const struct fetch_multipart_data *list)
 	return result;
 }
 
+
+/* exported interface documented in content/fetch.h */
+const char *
+fetch_multipart_data_find(const struct fetch_multipart_data *list,
+			  const char *name)
+{
+	while (list != NULL) {
+		if (strcmp(list->name, name) == 0) {
+			return list->value;
+		}
+		list = list->next;
+	}
+
+	return NULL;
+}
+
+
 /* exported interface documented in content/fetch.h */
 void fetch_multipart_data_destroy(struct fetch_multipart_data *list)
 {
@@ -726,17 +748,58 @@ void fetch_multipart_data_destroy(struct fetch_multipart_data *list)
 		free(list->name);
 		free(list->value);
 		if (list->file) {
-			FETCH_LOG("Freeing rawfile: %s", list->rawfile);
+			NSLOG(fetch, DEBUG,
+			      "Freeing rawfile: %s", list->rawfile);
 			free(list->rawfile);
 		}
 		free(list);
 	}
 }
 
+
+/* exported interface documented in content/fetch.h */
+nserror
+fetch_multipart_data_new_kv(struct fetch_multipart_data **list,
+			    const char *name,
+			    const char *value)
+{
+	struct fetch_multipart_data *newdata;
+
+	assert(list);
+
+	newdata = calloc(sizeof(*newdata), 1);
+
+	if (newdata == NULL) {
+		return NSERROR_NOMEM;
+	}
+
+	newdata->name = strdup(name);
+	if (newdata->name == NULL) {
+		free(newdata);
+		return NSERROR_NOMEM;
+	}
+
+	newdata->value = strdup(value);
+	if (newdata->value == NULL) {
+		free(newdata->name);
+		free(newdata);
+		return NSERROR_NOMEM;
+	}
+
+	newdata->next = *list;
+	*list = newdata;
+
+	return NSERROR_OK;
+}
+
+
 /* exported interface documented in content/fetch.h */
 void
 fetch_send_callback(const fetch_msg *msg, struct fetch *fetch)
 {
+	/* Bump the last_msg to the greatest seen msg */
+	if (msg->type > fetch->last_msg)
+		fetch->last_msg = msg->type;
 	fetch->callback(msg, fetch->p);
 }
 
@@ -744,8 +807,13 @@ fetch_send_callback(const fetch_msg *msg, struct fetch *fetch)
 /* exported interface documented in content/fetch.h */
 void fetch_remove_from_queues(struct fetch *fetch)
 {
-	FETCH_LOG("Fetch %p, fetcher %p can be freed",
-		   fetch, fetch->fetcher_handle);
+	int all_active;
+	int all_queued;
+
+	NSLOG(fetch, DEBUG,
+	      "Fetch %p, fetcher %p can be freed",
+	      fetch,
+	      fetch->fetcher_handle);
 
 	/* Go ahead and free the fetch properly now */
 	if (fetch->fetch_is_active) {
@@ -754,24 +822,19 @@ void fetch_remove_from_queues(struct fetch *fetch)
 		RING_REMOVE(queue_ring, fetch);
 	}
 
-#ifdef DEBUG_FETCH_VERBOSE
-	int all_active;
-	int all_queued;
 
 	RING_GETSIZE(struct fetch, fetch_ring, all_active);
 	RING_GETSIZE(struct fetch, queue_ring, all_queued);
 
-	LOG("Fetch ring is now %d elements.", all_active);
-
-	LOG("Queue ring is now %d elements.", all_queued);
-#endif
+	NSLOG(fetch, DEBUG, "Fetch ring is now %d elements.", all_active);
+	NSLOG(fetch, DEBUG, "Queue ring is now %d elements.", all_queued);
 }
 
 
 /* exported interface documented in content/fetch.h */
 void fetch_set_http_code(struct fetch *fetch, long http_code)
 {
-	FETCH_LOG("Setting HTTP code to %ld", http_code);
+	NSLOG(fetch, DEBUG, "Setting HTTP code to %ld", http_code);
 
 	fetch->http_code = http_code;
 }
