@@ -36,7 +36,8 @@
 #include "content/hlcache.h"
 
 #include "html/box.h"
-#include "html/html_internal.h"
+#include "html/box_construct.h"
+#include "html/private.h"
 #include "html/imagemap.h"
 
 #define HASH_SIZE 31 /* fixed size hash table */
