@@ -30,9 +30,11 @@
 
 #include "utils/messages.h"
 #include "utils/utils.h"
+#include "utils/nsurl.h"
 #include "netsurf/plotters.h"
 #include "netsurf/content.h"
 #include "content/content_protected.h"
+#include "content/content_factory.h"
 
 #include "image/svg.h"
 
@@ -59,7 +61,7 @@ static nserror svg_create_svg_data(svg_content *c)
 	return NSERROR_OK;
 
 no_memory:
-	content_broadcast_errorcode(&c->base, NSERROR_NOMEM);
+	content_broadcast_error(&c->base, NSERROR_NOMEM, NULL);
 	return NSERROR_NOMEM;
 }
 
