@@ -31,60 +31,23 @@
 #include "utils/log.h"
 #include "utils/utils.h"
 #include "netsurf/layout.h"
-#include "netsurf/plotters.h"
 #include "netsurf/content.h"
+#include "netsurf/window.h"
+#include "netsurf/browser_window.h"
 #include "content/hlcache.h"
 #include "content/urldb.h"
 #include "netsurf/bitmap.h"
+#include "utils/corestrings.h"
 
 #include "desktop/gui_internal.h"
-#include "desktop/browser_history.h"
 #include "desktop/browser_private.h"
-
-#define WIDTH 100
-#define HEIGHT 86
-#define RIGHT_MARGIN 50
-#define BOTTOM_MARGIN 30
-
-struct history_page {
-	nsurl *url;    /**< Page URL, never 0. */
-	lwc_string *frag_id; /** Fragment identifier, or 0. */
-	char *title;  /**< Page title, never 0. */
-};
-
-/** A node in the history tree. */
-struct history_entry {
-  	struct history_page page;
-	struct history_entry *back;  /**< Parent. */
-	struct history_entry *next;  /**< Next sibling. */
-	struct history_entry *forward;  /**< First child. */
-	struct history_entry *forward_pref;  /**< Child in direction of
-	                                          current entry. */
-	struct history_entry *forward_last;  /**< Last child. */
-	unsigned int children;  /**< Number of children. */
-	int x;  /**< Position of node. */
-	int y;  /**< Position of node. */
-	struct bitmap *bitmap;  /**< Thumbnail bitmap, or 0. */
-};
-
-/** History tree for a window. */
-struct history {
-	/** First page in tree (page that window opened with). */
-	struct history_entry *start;
-	/** Current position in tree. */
-	struct history_entry *current;
-	/** Width of layout. */
-	int width;
-	/** Height of layout. */
-	int height;
-};
-
+#include "desktop/browser_history.h"
 
 /**
  * Clone a history entry
  *
  * \param history opaque history structure, as returned by history_create()
- * \param entry   entry to clone
+ * \param entry entry to clone
  * \return A cloned history entry or NULL on error
  */
 static struct history_entry *
@@ -96,47 +59,87 @@ browser_window_history__clone_entry(struct history *history,
 	struct history_entry *prev = NULL;
 	struct history_entry *new_entry;
 
+	assert(entry);
 	assert(entry->page.url);
 	assert(entry->page.title);
 
 	/* clone the entry */
-	new_entry = malloc(sizeof *entry);
-	if (!new_entry)
+	new_entry = calloc(1, sizeof *entry);
+	if (!new_entry) {
 		return NULL;
+	}
 
-	memcpy(new_entry, entry, sizeof *entry);
-	new_entry->page.url = nsurl_ref(entry->page.url);
-	if (entry->page.frag_id)
-		new_entry->page.frag_id = lwc_string_ref(entry->page.frag_id);
-
+	/* copy page information */
 	new_entry->page.title = strdup(entry->page.title);
-	if (!new_entry->page.url || !new_entry->page.title ||
-			((entry->page.frag_id) && (!new_entry->page.frag_id))) {
-		nsurl_unref(new_entry->page.url);
-		if (new_entry->page.frag_id)
-			lwc_string_unref(new_entry->page.frag_id);
+	if (new_entry->page.title == NULL) {
+		free(new_entry);
+		return NULL;
+	}
+
+	new_entry->page.url = nsurl_ref(entry->page.url);
+	if (new_entry->page.url == NULL) {
 		free(new_entry->page.title);
 		free(new_entry);
 		return NULL;
 	}
 
-	/* update references */
-	if (history->current == entry)
-		history->current = new_entry;
-
-	/* recurse for all children */
-	for (child = new_entry->forward; child; child = child->next) {
-		new_child = browser_window_history__clone_entry(history, child);
-		if (new_child) {
-			new_child->back = new_entry;
-		} else {
+	if (entry->page.frag_id == NULL) {
+		new_entry->page.frag_id = NULL;
+	} else {
+		new_entry->page.frag_id = lwc_string_ref(entry->page.frag_id);
+		if (new_entry->page.frag_id == NULL) {
 			nsurl_unref(new_entry->page.url);
-			if (new_entry->page.frag_id)
-				lwc_string_unref(new_entry->page.frag_id);
-			free(new_entry->page.title);
 			free(new_entry);
 			return NULL;
 		}
+	}
+
+	if (entry->page.bitmap == NULL) {
+		new_entry->page.bitmap = NULL;
+	} else {
+		/* create a new bitmap and copy original into it */
+		unsigned char *bmsrc_data;
+		unsigned char *bmdst_data;
+		size_t bmsize;
+
+		new_entry->page.bitmap = guit->bitmap->create(
+				LOCAL_HISTORY_WIDTH,
+				LOCAL_HISTORY_HEIGHT,
+				BITMAP_NEW | BITMAP_OPAQUE);
+
+		if (new_entry->page.bitmap != NULL) {
+			bmsrc_data = guit->bitmap->get_buffer(entry->page.bitmap);
+			bmdst_data = guit->bitmap->get_buffer(new_entry->page.bitmap);
+			bmsize = guit->bitmap->get_rowstride(new_entry->page.bitmap) *
+				guit->bitmap->get_height(new_entry->page.bitmap);
+			memcpy(bmdst_data, bmsrc_data, bmsize);
+		}
+	}
+
+	/* copy tree values */
+	new_entry->back = entry->back;
+	new_entry->next = entry->next;
+	new_entry->forward = entry->forward;
+	new_entry->forward_pref = entry->forward_pref;
+	new_entry->forward_last = entry->forward_last;
+
+	/* recurse for all children */
+	for (child = new_entry->forward; child != NULL; child = child->next) {
+		new_child = browser_window_history__clone_entry(history, child);
+		if (new_child == NULL) {
+			nsurl_unref(new_entry->page.url);
+			if (new_entry->page.frag_id) {
+				lwc_string_unref(new_entry->page.frag_id);
+			}
+			free(new_entry->page.title);
+			if (entry->page.bitmap != NULL) {
+				guit->bitmap->destroy(entry->page.bitmap);
+			}
+			free(new_entry);
+			return NULL;
+		}
+
+		new_child->back = new_entry;
 		if (prev)
 			prev->next = new_child;
 		if (new_entry->forward == child)
@@ -147,6 +150,12 @@ browser_window_history__clone_entry(struct history *history,
 			new_entry->forward_last = new_child;
 		prev = new_child;
 	}
+
+	/* update references */
+	if (history->current == entry) {
+		history->current = new_entry;
+	}
+
 	return new_entry;
 }
 
@@ -157,15 +166,20 @@ browser_window_history__clone_entry(struct history *history,
 
 static void browser_window_history__free_entry(struct history_entry *entry)
 {
-	if (!entry)
-		return;
-	browser_window_history__free_entry(entry->forward);
-	browser_window_history__free_entry(entry->next);
-	nsurl_unref(entry->page.url);
-	if (entry->page.frag_id)
-		lwc_string_unref(entry->page.frag_id);
-	free(entry->page.title);
-	free(entry);
+	if (entry != NULL) {
+		browser_window_history__free_entry(entry->forward);
+		browser_window_history__free_entry(entry->next);
+
+		nsurl_unref(entry->page.url);
+		if (entry->page.frag_id) {
+			lwc_string_unref(entry->page.frag_id);
+		}
+		free(entry->page.title);
+		if (entry->page.bitmap != NULL) {
+			guit->bitmap->destroy(entry->page.bitmap);
+		}
+		free(entry);
+	}
 }
 
 
@@ -185,26 +199,26 @@ static int browser_window_history__layout_subtree(struct history *history,
 	struct history_entry *child;
 	int y1 = y;
 
-	if (history->width < x + WIDTH)
-		history->width = x + WIDTH;
+	if (history->width < x + LOCAL_HISTORY_WIDTH)
+		history->width = x + LOCAL_HISTORY_WIDTH;
 
 	if (!entry->forward) {
 		entry->x = x;
 		entry->y = y;
-		return y + HEIGHT;
+		return y + LOCAL_HISTORY_HEIGHT;
 	}
 
 	/* layout child subtrees below each other */
 	for (child = entry->forward; child; child = child->next) {
 		y1 = browser_window_history__layout_subtree(history, child,
-				x + WIDTH + RIGHT_MARGIN, y1);
+				x + LOCAL_HISTORY_WIDTH + LOCAL_HISTORY_RIGHT_MARGIN, y1);
 		if (child->next)
-			y1 += BOTTOM_MARGIN;
+			y1 += LOCAL_HISTORY_BOTTOM_MARGIN;
 	}
 
 	/* place ourselves in the middle */
 	entry->x = x;
-	entry->y = (y + y1) / 2 - HEIGHT / 2;
+	entry->y = (y + y1) / 2 - LOCAL_HISTORY_HEIGHT / 2;
 
 	return y1;
 }
@@ -227,152 +241,18 @@ static void browser_window_history__layout(struct history *history)
 	if (history->start)
 		history->height = browser_window_history__layout_subtree(
 				history, history->start,
-				RIGHT_MARGIN / 2, BOTTOM_MARGIN / 2);
+				LOCAL_HISTORY_RIGHT_MARGIN / 2,
+				LOCAL_HISTORY_BOTTOM_MARGIN / 2);
 	else
 		history->height = 0;
 
-	history->width += RIGHT_MARGIN / 2;
-	history->height += BOTTOM_MARGIN / 2;
-}
-
-/**
- * Recursively redraw a history_entry.
- *
- * \param history history containing the entry
- * \param entry entry to render
- * \param x0 area top left x coordinate
- * \param y0 area top left y coordinate
- * \param x1 area bottom right x coordinate
- * \param y1 area bottom right y coordinate
- * \param x window x offset
- * \param y window y offset
- * \param clip clip redraw
- * \param ctx     current redraw context
- */
-static bool
-browser_window_history__redraw_entry(struct history *history,
-		struct history_entry *entry,
-		int x0, int y0, int x1, int y1,
-		int x, int y, bool clip,
-		const struct redraw_context *ctx)
-{
-	const struct plotter_table *plot = ctx->plot;
-	size_t char_offset;
-	int actual_x;
-	struct history_entry *child;
-	colour c = entry == history->current ?
-			HISTORY_COLOUR_SELECTED : HISTORY_COLOUR_FOREGROUND;
-	int tailsize = 5;
-	int xoffset = x - x0;
-	int yoffset = y - y0;
-        plot_style_t pstyle_history_rect = { 
-            .stroke_type = PLOT_OP_TYPE_SOLID,
-            .stroke_colour = c,
-            .stroke_width = entry == history->current ? 3 : 1,
-        };
-	plot_font_style_t fstyle = *plot_style_font;
-	nserror res;
-
-	if (clip) {
-		struct rect rect;
-		rect.x0 = x0 + xoffset;
-		rect.y0 = y0 + yoffset;
-		rect.x1 = x1 + xoffset;
-		rect.y1 = y1 + yoffset;
-		if (!plot->clip(&rect)) {
-			return false;
-		}
-	}
-
-	/* Only attempt to plot bitmap if it is present */
-	if (entry->bitmap != NULL) {
-		plot->bitmap(entry->x + xoffset,
-			     entry->y + yoffset,
-			     WIDTH, HEIGHT,
-			     entry->bitmap, 0xffffff, 0);
-	}
-
-	if (!plot->rectangle(entry->x - 1 + xoffset, 
-                            entry->y - 1 + yoffset,
-                            entry->x + xoffset + WIDTH, 
-                            entry->y + yoffset + HEIGHT,
-			     &pstyle_history_rect)) {
-		return false;
-	}
-
-	res = guit->layout->position(plot_style_font, entry->page.title,
-				     strlen(entry->page.title), WIDTH,
-				     &char_offset, &actual_x);
-	if (res != NSERROR_OK) {
-		return false;
-	}
-
-	fstyle.background = HISTORY_COLOUR_BACKGROUND;
-	fstyle.foreground = c;
-	fstyle.weight = entry == history->current ? 900 : 400;
-
-	if (!plot->text(entry->x + xoffset, entry->y + HEIGHT + 12 + yoffset,
-			entry->page.title, char_offset, &fstyle))
-		return false;
-
-	for (child = entry->forward; child; child = child->next) {
-		if (!plot->line(entry->x + WIDTH + xoffset,
-				entry->y + HEIGHT / 2 + yoffset,
-		      	entry->x + WIDTH + tailsize + xoffset,
-				entry->y + HEIGHT / 2 + yoffset, 
-			       plot_style_stroke_history))
-			return false;
-		if (!plot->line(entry->x + WIDTH + tailsize + xoffset,
-			       entry->y + HEIGHT / 2 + yoffset,
-			       child->x - tailsize +xoffset,
-			       child->y + HEIGHT / 2 + yoffset,
-			       plot_style_stroke_history))
-			return false;
-		if (!plot->line(child->x - tailsize + xoffset,
-			       child->y + HEIGHT / 2 + yoffset,
-			       child->x + xoffset, child->y +
-			       			HEIGHT / 2 + yoffset,
-			       plot_style_stroke_history))
-			return false;
-		if (!browser_window_history__redraw_entry(history, child,
-				x0, y0, x1, y1, x, y, clip, ctx))
-			return false;
-	}
-
-	return true;
+	history->width += LOCAL_HISTORY_RIGHT_MARGIN / 2;
+	history->height += LOCAL_HISTORY_BOTTOM_MARGIN / 2;
 }
 
 
-/**
- * Find the history entry at a position.
- *
- * \param  entry  entry to search from
- * \param  x      coordinate
- * \param  y      coordinate
- * \return  an entry if found, 0 if none
- */
 
-static struct history_entry *browser_window_history__find_position(
-		struct history_entry *entry, int x, int y)
-{
-	struct history_entry *child;
-	struct history_entry *found;
 
-	if (!entry)
-		return 0;
-
-	if (entry->x <= x && x <= entry->x + WIDTH &&
-			entry->y <= y && y <= entry->y + HEIGHT)
-		return entry;
-
-	for (child = entry->forward; child; child = child->next) {
-		found = browser_window_history__find_position(child, x, y);
-		if (found)
-			return found;
-	}
-
-	return 0;
-}
 
 /**
  * Enumerate subentries in history
@@ -393,7 +273,8 @@ static bool browser_window_history__enumerate_entry(
 	const struct history_entry *child;
 
 	if (!cb(bw, entry->x, entry->y,
-			entry->x + WIDTH, entry->y + HEIGHT,
+			entry->x + LOCAL_HISTORY_WIDTH,
+			entry->y + LOCAL_HISTORY_HEIGHT,
 			entry, ud))
 		return false;
 
@@ -422,10 +303,11 @@ nserror browser_window_history_create(struct browser_window *bw)
 		return NSERROR_NOMEM;
 	}
 
-	history->width = RIGHT_MARGIN / 2;
-	history->height = BOTTOM_MARGIN / 2;
+	history->width = LOCAL_HISTORY_RIGHT_MARGIN / 2;
+	history->height = LOCAL_HISTORY_BOTTOM_MARGIN / 2;
 
 	bw->history = history;
+
 	return NSERROR_OK;
 }
 
@@ -454,7 +336,7 @@ nserror browser_window_history_clone(const struct browser_window *existing,
 	new_history->start = browser_window_history__clone_entry(new_history,
 			new_history->start);
 	if (!new_history->start) {
-		LOG("Insufficient memory to clone history");
+		NSLOG(netsurf, INFO, "Insufficient memory to clone history");
 		browser_window_history_destroy(clone);
 		clone->history = NULL;
 		return NSERROR_NOMEM;
@@ -465,14 +347,14 @@ nserror browser_window_history_clone(const struct browser_window *existing,
 
 
 /* exported interface documented in desktop/browser_history.h */
-nserror browser_window_history_add(struct browser_window *bw,
-		struct hlcache_handle *content, lwc_string *frag_id)
+nserror
+browser_window_history_add(struct browser_window *bw,
+			   struct hlcache_handle *content,
+			   lwc_string *frag_id)
 {
 	struct history *history;
 	struct history_entry *entry;
-	nsurl *nsurl = hlcache_handle_get_url(content);
 	char *title;
-	struct bitmap *bitmap;
 	nserror ret;
 
 	assert(bw);
@@ -481,32 +363,51 @@ nserror browser_window_history_add(struct browser_window *bw,
 
 	history = bw->history;
 
-	/* allocate space */
 	entry = malloc(sizeof *entry);
 	if (entry == NULL) {
 		return NSERROR_NOMEM;
 	}
 
+	/* page information */
 	title = strdup(content_get_title(content));
 	if (title == NULL) {
 		free(entry);
 		return NSERROR_NOMEM;
 	}
 
-	entry->page.url = nsurl_ref(nsurl);
-	entry->page.frag_id = frag_id ? lwc_string_ref(frag_id) : 0;
-
+	entry->page.url = nsurl_ref(hlcache_handle_get_url(content));
+	entry->page.frag_id = frag_id ? lwc_string_ref(frag_id) : NULL;
 	entry->page.title = title;
+	entry->page.scroll_x = 0.0f;
+	entry->page.scroll_y = 0.0f;
+
+	/* create thumbnail for localhistory view */
+	NSLOG(netsurf, DEBUG,
+	      "Creating thumbnail for %s", nsurl_access(entry->page.url));
+
+	entry->page.bitmap = guit->bitmap->create(
+			LOCAL_HISTORY_WIDTH, LOCAL_HISTORY_HEIGHT,
+			BITMAP_NEW | BITMAP_CLEAR_MEMORY | BITMAP_OPAQUE);
+	if (entry->page.bitmap != NULL) {
+		ret = guit->bitmap->render(entry->page.bitmap, content);
+		if (ret != NSERROR_OK) {
+			/* Thumbnail render failed */
+			NSLOG(netsurf, WARNING, "Thumbnail render failed");
+		}
+	}
+
+	/* insert into tree */
 	entry->back = history->current;
-	entry->next = 0;
-	entry->forward = entry->forward_pref = entry->forward_last = 0;
+	entry->next = NULL;
+	entry->forward = entry->forward_pref = entry->forward_last = NULL;
 	entry->children = 0;
-	entry->bitmap = 0;
+
 	if (history->current) {
-		if (history->current->forward_last)
+		if (history->current->forward_last) {
 			history->current->forward_last->next = entry;
-		else
+		} else {
 			history->current->forward = entry;
+		}
 		history->current->forward_pref = entry;
 		history->current->forward_last = entry;
 		history->current->children++;
@@ -515,33 +416,6 @@ nserror browser_window_history_add(struct browser_window *bw,
 	}
 	history->current = entry;
 
-	/* if we have a thumbnail, don't update until the page has finished
-	 * loading */
-	bitmap = urldb_get_thumbnail(nsurl);
-	if (bitmap == NULL) {
-		LOG("Creating thumbnail for %s", nsurl_access(nsurl));
-		bitmap = guit->bitmap->create(WIDTH, HEIGHT,
-					      BITMAP_NEW | BITMAP_CLEAR_MEMORY |
-					      BITMAP_OPAQUE);
-		if (bitmap != NULL) {
-			ret = guit->bitmap->render(bitmap, content);
-			if (ret == NSERROR_OK) {
-				/* Successful thumbnail so register it
-				 * with the url.
-				 */
-				urldb_set_thumbnail(nsurl, bitmap);
-			} else {
-				/* Thumbnailing failed. Ignore it
-				 * silently but clean up bitmap.
-				 */
-				LOG("Thumbnail renderfailed");
-				guit->bitmap->destroy(bitmap);
-				bitmap = NULL;
-			}
-		}
-	}
-	entry->bitmap = bitmap;
-
 	browser_window_history__layout(history);
 
 	return NSERROR_OK;
@@ -554,12 +428,15 @@ nserror browser_window_history_update(struct browser_window *bw,
 {
 	struct history *history;
 	char *title;
+	int sx, sy;
 
 	assert(bw != NULL);
 
 	history = bw->history;
 
-	if (!history || !history->current || !history->current->bitmap) {
+	if (!history ||
+	    !history->current ||
+	    !history->current->page.bitmap) {
 		return NSERROR_INVALID;
 	}
 
@@ -570,16 +447,59 @@ nserror browser_window_history_update(struct browser_window *bw,
 	if (title == NULL) {
 		return NSERROR_NOMEM;
 	}
-
+	NSLOG(netsurf, INFO, "Updating history entry for %s", title);
 	free(history->current->page.title);
 	history->current->page.title = title;
 
-	guit->bitmap->render(history->current->bitmap, content);
+	if (history->current->page.bitmap != NULL) {
+		guit->bitmap->render(history->current->page.bitmap, content);
+	}
 
+	if (bw->window != NULL &&
+	    guit->window->get_scroll(bw->window, &sx, &sy)) {
+		int content_height = content_get_height(content);
+		int content_width = content_get_width(content);
+		/* clamp width and height values */
+		if (content_height < 1) {
+			content_height = 1;
+		}
+		if (content_width < 1) {
+			content_width = 1;
+		}
+		/* Successfully got scroll offsets, update the entry */
+		history->current->page.scroll_x = \
+			(float)sx / (float)content_width;
+		history->current->page.scroll_y = \
+			(float)sy / (float)content_height;
+		NSLOG(netsurf, INFO, "Updated scroll offsets to %g by %g",
+		      history->current->page.scroll_x,
+		      history->current->page.scroll_y);
+	}
 	return NSERROR_OK;
 }
 
+/* exported interface documented in desktop/browser_private.h */
+nserror
+browser_window_history_get_scroll(struct browser_window *bw,
+				  float *sx, float *sy)
+{
+	struct history *history;
 
+	assert(bw != NULL);
+
+	history = bw->history;
+
+	if (!history ||
+	    !history->current ||
+	    !history->current->page.bitmap) {
+		return NSERROR_INVALID;
+	}
+
+	*sx = history->current->page.scroll_x;
+	*sy = history->current->page.scroll_y;
+
+	return NSERROR_OK;
+}
 
 /* exported interface documented in desktop/browser_history.h */
 void browser_window_history_destroy(struct browser_window *bw)
@@ -600,6 +520,24 @@ void browser_window_history_destroy(struct browser_window *bw)
 /* exported interface documented in desktop/browser_history.h */
 nserror browser_window_history_back(struct browser_window *bw, bool new_window)
 {
+	if (bw != NULL && bw->internal_nav) {
+		/* All internal nav back operations ignore new_window */
+		if (bw->current_parameters.url != NULL) {
+			/* There are some internal parameters, restart from there */
+			return browser_window__reload_current_parameters(bw);
+		} else {
+			/* No internal parameters, just navigate to about:blank */
+			return browser_window_navigate(
+				bw,
+				corestring_nsurl_about_blank,
+				NULL, /* Referer */
+				BW_NAVIGATE_HISTORY,
+				NULL, /* Post */
+				NULL, /* Post */
+				NULL  /* parent fetch */);
+		}
+	}
+
 	if (!bw || !bw->history || !bw->history->current ||
 	    !bw->history->current->back) {
 		return NSERROR_BAD_PARAMETER;
@@ -638,6 +576,27 @@ bool browser_window_history_forward_available(struct browser_window *bw)
 			bw->history->current->forward_pref);
 }
 
+/* exported interface documented in desktop/browser_history.h */
+nserror
+browser_window_history_get_thumbnail(struct browser_window *bw,
+				 struct bitmap **bitmap_out)
+{
+	struct bitmap *bitmap;
+
+	if (!bw || !bw->history || !bw->history->current) {
+		return NSERROR_INVALID;
+	}
+
+	if (bw->history->current->page.bitmap == NULL) {
+		bitmap = content_get_bitmap(bw->current_content);
+	} else {
+		bitmap = bw->history->current->page.bitmap;
+	}
+
+	*bitmap_out = bitmap;
+
+	return NSERROR_OK;
+}
 
 /* exported interface documented in desktop/browser_history.h */
 nserror browser_window_history_go(struct browser_window *bw,
@@ -670,9 +629,13 @@ nserror browser_window_history_go(struct browser_window *bw,
 				url, NULL, bw, NULL);
 		history->current = current;
 	} else {
+		if (bw->current_content != NULL) {
+			browser_window_history_update(bw, bw->current_content);
+		}
 		history->current = entry;
 		error = browser_window_navigate(bw, url, NULL,
-				BW_NAVIGATE_NONE, NULL, NULL, NULL);
+				BW_NAVIGATE_NO_TERMINAL_HISTORY_UPDATE,
+				NULL, NULL, NULL);
 	}
 
 	nsurl_unref(url);
@@ -682,97 +645,7 @@ nserror browser_window_history_go(struct browser_window *bw,
 
 
 /* exported interface documented in desktop/browser_history.h */
-void browser_window_history_size(struct browser_window *bw,
-		int *width, int *height)
-{
-	assert(bw != NULL);
-	assert(bw->history != NULL);
-
-	*width = bw->history->width;
-	*height = bw->history->height;
-}
-
-
-/* exported interface documented in desktop/browser_history.h */
-bool browser_window_history_redraw(struct browser_window *bw,
-		const struct redraw_context *ctx)
-{
-	struct history *history;
-
-	assert(bw != NULL);
-	history = bw->history;
-
-	if (history == NULL) {
-		LOG("Attempt to draw NULL history.");
-		return false;
-	}
-
-	if (!history->start)
-		return true;
-	return browser_window_history__redraw_entry(history, history->start,
-			0, 0, 0, 0, 0, 0, false, ctx);
-}
-
-
-/* exported interface documented in desktop/browser_history.h */
-bool browser_window_history_redraw_rectangle(struct browser_window *bw,
-	int x0, int y0, int x1, int y1,
-	int x, int y, const struct redraw_context *ctx)
-{
-	struct history *history;
-
-	assert(bw != NULL);
-	history = bw->history;
-
-	if (!history->start)
-		return true;
-	return browser_window_history__redraw_entry(history, history->start,
-		x0, y0, x1, y1, x, y, true, ctx);
-}
-
-
-/* exported interface documented in desktop/browser_history.h */
-bool browser_window_history_click(struct browser_window *bw,
-		int x, int y, bool new_window)
-{
-	struct history_entry *entry;
-	struct history *history;
-
-	assert(bw != NULL);
-	history = bw->history;
-
-	entry = browser_window_history__find_position(history->start, x, y);
-	if (!entry)
-		return false;
-	if (entry == history->current)
-		return false;
-
-	browser_window_history_go(bw, entry, new_window);
-
-	return true;
-}
-
-
-/* exported interface documented in desktop/browser_history.h */
-const char *browser_window_history_position_url(struct browser_window *bw,
-		int x, int y)
-{
-	struct history_entry *entry;
-	struct history *history;
-
-	assert(bw != NULL);
-	history = bw->history;
-
-	entry = browser_window_history__find_position(history->start, x, y);
-	if (!entry)
-		return 0;
-
-	return nsurl_access(entry->page.url);
-}
-
-
-/* exported interface documented in desktop/browser_history.h */
-void browser_window_history_enumerate_forward(const struct browser_window *bw, 
+void browser_window_history_enumerate_forward(const struct browser_window *bw,
 		browser_window_history_enumerate_cb cb, void *user_data)
 {
 	struct history_entry *e;
@@ -782,7 +655,8 @@ void browser_window_history_enumerate_forward(const struct browser_window *bw,
 
 	e = bw->history->current->forward_pref;
 	for (; e != NULL; e = e->forward_pref) {
-		if (!cb(bw, e->x, e->y, e->x + WIDTH, e->y + HEIGHT,
+		if (!cb(bw, e->x, e->y, e->x + LOCAL_HISTORY_WIDTH,
+				e->y + LOCAL_HISTORY_HEIGHT,
 				e, user_data))
 			break;
 	}
@@ -790,7 +664,7 @@ void browser_window_history_enumerate_forward(const struct browser_window *bw,
 
 
 /* exported interface documented in desktop/browser_history.h */
-void browser_window_history_enumerate_back(const struct browser_window *bw, 
+void browser_window_history_enumerate_back(const struct browser_window *bw,
 		browser_window_history_enumerate_cb cb, void *user_data)
 {
 	struct history_entry *e;
@@ -799,7 +673,8 @@ void browser_window_history_enumerate_back(const struct browser_window *bw,
 		return;
 
 	for (e = bw->history->current->back; e != NULL; e = e->back) {
-		if (!cb(bw, e->x, e->y, e->x + WIDTH, e->y + HEIGHT,
+		if (!cb(bw, e->x, e->y, e->x + LOCAL_HISTORY_WIDTH,
+				e->y + LOCAL_HISTORY_HEIGHT,
 				e, user_data))
 			break;
 	}
@@ -818,10 +693,9 @@ void browser_window_history_enumerate(const struct browser_window *bw,
 
 
 /* exported interface documented in desktop/browser_history.h */
-const char *browser_window_history_entry_get_url(
-		const struct history_entry *entry)
+nsurl *browser_window_history_entry_get_url(const struct history_entry *entry)
 {
-	return nsurl_access(entry->page.url);
+	return nsurl_ref(entry->page.url);
 }
 
 
