@@ -40,6 +40,7 @@
 #endif
 
 #ifdef __amigaos4__
+#include <exec/extmem.h>
 #include <sys/param.h>
 #endif
 #include "assert.h"
@@ -54,8 +55,10 @@
 #include "amiga/gui.h"
 #include "amiga/bitmap.h"
 #include "amiga/plotters.h"
+#include "amiga/memory.h"
 #include "amiga/misc.h"
 #include "amiga/rtg.h"
+#include "amiga/schedule.h"
 
 // disable use of "triangle mode" for scaling
 #ifdef AMI_NS_TRIANGLE_SCALING
@@ -66,6 +69,8 @@ struct bitmap {
 	int width;
 	int height;
 	UBYTE *pixdata;
+	struct ExtMemIFace *iextmem;
+	uint32 size;
 	bool opaque;
 	int native;
 	struct BitMap *nativebm;
@@ -107,12 +112,30 @@ void *amiga_bitmap_create(int width, int height, unsigned int state)
 {
 	struct bitmap *bitmap;
 
-	if(pool_bitmap == NULL) pool_bitmap = ami_misc_itempool_create(sizeof(struct bitmap));
+	if(pool_bitmap == NULL) pool_bitmap = ami_memory_itempool_create(sizeof(struct bitmap));
 
-	bitmap = ami_misc_itempool_alloc(pool_bitmap, sizeof(struct bitmap));
+	bitmap = ami_memory_itempool_alloc(pool_bitmap, sizeof(struct bitmap));
 	if(bitmap == NULL) return NULL;
 
-	bitmap->pixdata = ami_misc_allocvec_clear(width*height*4, 0xff);
+	bitmap->size = width * height * 4;
+
+#ifdef __amigaos4__
+	if(nsoption_bool(use_extmem) == true) {
+		uint64 size64 = bitmap->size;
+		bitmap->iextmem = AllocSysObjectTags(ASOT_EXTMEM,
+								ASOEXTMEM_Size, &size64,
+								ASOEXTMEM_AllocationPolicy, EXTMEMPOLICY_IMMEDIATE,
+								TAG_END);
+
+		bitmap->pixdata = NULL;
+		UBYTE *pixdata = amiga_bitmap_get_buffer(bitmap);
+		memset(pixdata, 0xff, bitmap->size);
+	} else
+#endif
+	{
+		bitmap->pixdata = ami_memory_clear_alloc(bitmap->size, 0xff);
+	}
+
 	bitmap->width = width;
 	bitmap->height = height;
 
@@ -132,11 +155,42 @@ void *amiga_bitmap_create(int width, int height, unsigned int state)
 	return bitmap;
 }
 
+static void amiga_bitmap_unmap_buffer(void *p)
+{
+#ifdef __amigaos4__
+	struct bitmap *bm = p;
+
+	if((nsoption_bool(use_extmem) == true) && (bm->pixdata != NULL)) {
+		NSLOG(netsurf, INFO,
+		      "Unmapping ExtMem object %p for bitmap %p",
+		      bm->iextmem,
+		      bm);
+		bm->iextmem->Unmap(bm->pixdata, bm->size);
+		bm->pixdata = NULL;
+	}
+#endif
+}
 
 /* exported function documented in amiga/bitmap.h */
 unsigned char *amiga_bitmap_get_buffer(void *bitmap)
 {
 	struct bitmap *bm = bitmap;
+
+#ifdef __amigaos4__
+	if(nsoption_bool(use_extmem) == true) {
+		if(bm->pixdata == NULL) {
+			NSLOG(netsurf, INFO,
+			      "Mapping ExtMem object %p for bitmap %p",
+			      bm->iextmem,
+			      bm);
+			bm->pixdata = bm->iextmem->Map(NULL, bm->size, 0LL, 0);
+		}
+
+		/* unmap the buffer after one second */
+		ami_schedule(1000, amiga_bitmap_unmap_buffer, bm);
+	}
+#endif
+
 	return bm->pixdata;
 }
 
@@ -168,8 +222,19 @@ void amiga_bitmap_destroy(void *bitmap)
 		}
 
 		if(bm->native_mask) FreeRaster(bm->native_mask, bm->width, bm->height);
-		if(bm->drawhandle) ReleaseDrawHandle(bm->drawhandle);
-		FreeVec(bm->pixdata);
+
+#ifdef __amigaos4__
+		if(nsoption_bool(use_extmem) == true) {
+			ami_schedule(-1, amiga_bitmap_unmap_buffer, bm);
+			amiga_bitmap_unmap_buffer(bm);
+			FreeSysObject(ASOT_EXTMEM, bm->iextmem);
+			bm->iextmem = NULL;
+		} else
+#endif
+		{
+			if(bm->drawhandle) ReleaseDrawHandle(bm->drawhandle);
+			ami_memory_clear_free(bm->pixdata);
+		}
 
 		if(bm->url) nsurl_unref(bm->url);
 		if(bm->title) free(bm->title);
@@ -181,7 +246,7 @@ void amiga_bitmap_destroy(void *bitmap)
 		bm->url = NULL;
 		bm->title = NULL;
 	
-		ami_misc_itempool_free(pool_bitmap, bm, sizeof(struct bitmap));
+		ami_memory_itempool_free(pool_bitmap, bm, sizeof(struct bitmap));
 		bm = NULL;
 	}
 }
@@ -217,9 +282,12 @@ void amiga_bitmap_modified(void *bitmap)
 {
 	struct bitmap *bm = bitmap;
 
-	if((bm->nativebm)) // && (bm->native == AMI_NSBM_TRUECOLOUR))
-		ami_rtg_freebitmap(bm->nativebm);
-		
+#ifdef __amigaos4__
+		/* unmap the buffer after 0.5s - we might need it imminently */
+		ami_schedule(500, amiga_bitmap_unmap_buffer, bm);
+#endif
+
+	if(bm->nativebm) ami_rtg_freebitmap(bm->nativebm);
 	if(bm->drawhandle) ReleaseDrawHandle(bm->drawhandle);
 	if(bm->native_mask) FreeRaster(bm->native_mask, bm->width, bm->height);
 	bm->nativebm = NULL;
@@ -422,6 +490,7 @@ static inline struct BitMap *ami_bitmap_get_generic(struct bitmap *bitmap,
 			int width, int height, struct BitMap *restrict friendbm, int type)
 {
 	struct BitMap *restrict tbm = NULL;
+	struct Screen *scrn = ami_gui_get_screen();
 
 	if(bitmap->nativebm)
 	{
@@ -463,17 +532,22 @@ static inline struct BitMap *ami_bitmap_get_generic(struct bitmap *bitmap,
 				}
 
 				ami_bitmap_rgba_to_argb(bitmap);
-				bitmap->drawhandle = ObtainDrawHandle(NULL,
-										&rp, scrn->ViewPort.ColorMap,
-										GGFX_DitherMode, dithermode,
-										TAG_DONE);
+				bitmap->drawhandle = ObtainDrawHandle(
+					NULL,
+					&rp,
+					scrn->ViewPort.ColorMap,
+					GGFX_DitherMode, dithermode,
+					TAG_DONE);
+				if(bitmap->drawhandle) {
+					APTR ddh = CreateDirectDrawHandle(bitmap->drawhandle,
+											bitmap->width, bitmap->height,
+											width, height, NULL);
 
-				APTR ddh = CreateDirectDrawHandle(bitmap->drawhandle,
-										bitmap->width, bitmap->height,
-										width, height, NULL);
-
-				DirectDrawTrueColor(ddh, (ULONG *)amiga_bitmap_get_buffer(bitmap), 0, 0, TAG_DONE);
-				DeleteDirectDrawHandle(ddh);
+					DirectDrawTrueColor(ddh, (ULONG *)amiga_bitmap_get_buffer(bitmap), 0, 0, TAG_DONE);
+					DeleteDirectDrawHandle(ddh);
+					ReleaseDrawHandle(bitmap->drawhandle);
+					bitmap->drawhandle = NULL;
+				}
 				ami_bitmap_argb_to_rgba(bitmap);
 			} else {
 				if(guigfx_warned == false) {
@@ -534,8 +608,12 @@ static inline struct BitMap *ami_bitmap_get_generic(struct bitmap *bitmap,
 						TAG_DONE);
 
 			if (err != COMPERR_Success) {
-				LOG("Composite error %ld - falling back", err);
-				/* If it failed, do it again the way which works in software */
+				NSLOG(netsurf, INFO,
+				      "Composite error %ld - falling back",
+				      err);
+				/* If it failed, do it again the way
+				 *  which works in software
+				 */
 #else
 			{
 #endif
@@ -549,7 +627,8 @@ static inline struct BitMap *ami_bitmap_get_generic(struct bitmap *bitmap,
 						COMPTAG_FriendBitMap, scrn->RastPort.BitMap,
 						TAG_DONE);
 				/* If it still fails... it's non-fatal */
-				LOG("Fallback returned error %ld", err);
+				NSLOG(netsurf, INFO,
+				      "Fallback returned error %ld", err);
 			}
 		} else /* Do it the old-fashioned way.  This is pretty slow, even on OS4.1 */
 #endif
@@ -641,11 +720,11 @@ static inline struct BitMap *ami_bitmap_get_palettemapped(struct bitmap *bitmap,
 }
 
 struct BitMap *ami_bitmap_get_native(struct bitmap *bitmap,
-				int width, int height, struct BitMap *friendbm)
+				int width, int height, bool palette_mapped, struct BitMap *friendbm)
 {
 	if(bitmap == NULL) return NULL;
 
-	if(__builtin_expect(ami_plot_screen_is_palettemapped() == true, 0)) {
+	if(__builtin_expect(palette_mapped == true, 0)) {
 		return ami_bitmap_get_palettemapped(bitmap, width, height, friendbm);
 	} else {
 		return ami_bitmap_get_truecolour(bitmap, width, height, friendbm);
@@ -654,33 +733,32 @@ struct BitMap *ami_bitmap_get_native(struct bitmap *bitmap,
 
 void ami_bitmap_fini(void)
 {
-	if(pool_bitmap) ami_misc_itempool_delete(pool_bitmap);
+	if(pool_bitmap) ami_memory_itempool_delete(pool_bitmap);
 	pool_bitmap = NULL;
 }
 
 static nserror bitmap_render(struct bitmap *bitmap, struct hlcache_handle *content)
 {
 #ifdef __amigaos4__
-	struct redraw_context ctx = {
-		.interactive = false,
-		.background_images = true,
-		.plot = &amiplot
-	};
+	NSLOG(netsurf, INFO, "Entering bitmap_render");
 
 	int plot_width;
 	int plot_height;
-	struct gui_globals bm_globals;
-	struct gui_globals *temp_gg = glob;
+	struct gui_globals *bm_globals;
 
 	plot_width = MIN(content_get_width(content), bitmap->width);
 	plot_height = ((plot_width * bitmap->height) + (bitmap->width / 2)) /
 			bitmap->width;
 
-	ami_init_layers(&bm_globals, bitmap->width, bitmap->height, true);
-	bm_globals.shared_pens = NULL;
+	bm_globals = ami_plot_ra_alloc(bitmap->width, bitmap->height, true, false);
+	ami_clearclipreg(bm_globals);
 
-	glob = &bm_globals;
-	ami_clearclipreg(&bm_globals);
+	struct redraw_context ctx = {
+		.interactive = false,
+		.background_images = true,
+		.plot = &amiplot,
+		.priv = bm_globals
+	};
 
 	content_scaled_redraw(content, plot_width, plot_height, &ctx);
 
@@ -688,7 +766,7 @@ static nserror bitmap_render(struct bitmap *bitmap, struct hlcache_handle *conte
 					BLITA_SrcY, 0,
 					BLITA_Width, bitmap->width,
 					BLITA_Height, bitmap->height,
-					BLITA_Source, bm_globals.bm,
+					BLITA_Source, ami_plot_ra_get_bitmap(bm_globals),
 					BLITA_SrcType, BLITT_BITMAP,
 					BLITA_Dest, amiga_bitmap_get_buffer(bitmap),
 					BLITA_DestType, BLITT_ARGB32,
@@ -702,14 +780,8 @@ static nserror bitmap_render(struct bitmap *bitmap, struct hlcache_handle *conte
 	/**\todo In theory we should be able to move the bitmap to our native area
 		to try to avoid re-conversion (at the expense of memory) */
 
-	ami_free_layers(&bm_globals);
+	ami_plot_ra_free(bm_globals);
 	amiga_bitmap_set_opaque(bitmap, true);
-
-	/* Restore previous render area.  This is set when plotting starts,
-	 * but if bitmap_render is called *during* a browser render then
-	 * having an invalid pointer here causes NetSurf to crash.
-	 */
-	glob = temp_gg;
 #else
 #warning FIXME for OS3 (in current state none of bitmap_render can work!)
 #endif
@@ -729,16 +801,17 @@ void ami_bitmap_set_title(struct bitmap *bm, const char *title)
 	bm->title = strdup(title);
 }
 
-ULONG *ami_bitmap_get_icondata(struct bitmap *bm)
-{
-	return bm->icondata;
-}
-
 void ami_bitmap_set_icondata(struct bitmap *bm, ULONG *icondata)
 {
 	bm->icondata = icondata;
 }
 
+void ami_bitmap_free_icondata(struct bitmap *bm)
+{
+	if(bm->icondata) free(bm->icondata);
+	bm->icondata = NULL;
+}
+
 bool ami_bitmap_is_nativebm(struct bitmap *bm, struct BitMap *nbm)
 {
 	if(bm->nativebm == nbm) return true;
