@@ -34,11 +34,9 @@ struct dlnode
 	char *filename;
 };
 
-void ami_download_window_abort(struct gui_download_window *dw);
-BOOL ami_download_window_event(struct gui_download_window *dw);
 void ami_free_download_list(struct List *dllist);
 BOOL ami_download_check_overwrite(const char *file, struct Window *win, ULONG size);
+void ami_download_parse_backmsg(const char *backmsg);
 
 nserror gui_window_save_link(struct gui_window *g, struct nsurl *url, const char *title);
-
 #endif
