@@ -27,6 +27,7 @@
 #ifndef __amigaos4__
 
 #include <stdint.h>
+#include <dirent.h>
 
 #include <proto/exec.h>
 #include <proto/dos.h>
@@ -78,6 +79,8 @@
 #define GAUGEIA_Level		TAG_IGNORE
 #define IA_InBorder			TAG_IGNORE
 #define IA_Label			TAG_IGNORE
+#define LBNCA_SoftStyle		TAG_IGNORE
+#define LISTBROWSER_Striping	TAG_IGNORE
 #define SA_Compositing		TAG_IGNORE
 #define SBNA_Text			TAG_IGNORE
 #define SBNA_HintInfo		TAG_IGNORE
@@ -104,10 +107,15 @@
 #define RAWKEY_F8	0x57
 #define RAWKEY_F9	0x58
 #define RAWKEY_F10	0x59
+#define RAWKEY_F12  0x6F
 #define RAWKEY_HELP	0x5F
 #define RAWKEY_HOME	0x70
 #define RAWKEY_END	0x71
 
+/* New pens - these may not be equivalent */
+#define DISABLEDTEXTPEN HIGHLIGHTTEXTPEN
+#define TITLEPEN FILLPEN
+
 /* Other constants */
 #define BVS_DISPLAY BVS_NONE
 #define IDCMP_EXTENDEDMOUSE 0
@@ -117,6 +125,7 @@
 #define OFF_OPEN 0
 #define AFF_OTAG 0
 #define ML_SEPARATOR NM_BARLABEL
+#define LBS_ROWS 0
 
 /* Renamed structures */
 #define AnchorPathOld AnchorPath
@@ -249,6 +258,10 @@ char *ASPrintf(const char *fmt, ...);
 
 /* C */
 char *strlwr(char *str);
+int alphasort(const struct dirent **d1, const struct dirent **d2);
+int scandir(const char *dir, struct dirent ***namelist,
+  int (*filter)(const struct dirent *),
+  int (*compar)(const struct dirent **, const struct dirent **));
 #endif
 #endif
 
