@@ -117,11 +117,9 @@ void gtk_fetch_filetype_init(const char *mimefile)
 
 			type = ptr;
 
-			/* search for the first non-whitespace char or NUL or
+			/* search for the first whitespace char or NUL or
 			 * NL */
-			while (*ptr &&
-			       (!ascii_is_space(*ptr)) &&
-			       *ptr != '\n') {
+			while (*ptr && (!ascii_is_space(*ptr))) {
 				ptr++;
 			}
 
@@ -146,9 +144,7 @@ void gtk_fetch_filetype_init(const char *mimefile)
 				/* search for the first whitespace char or
 				 * NUL or NL which is the end of the ext.
 				 */
-				while (*ptr &&
-				       (!ascii_is_space(*ptr)) &&
-				       *ptr != '\n') {
+				while (*ptr && (!ascii_is_space(*ptr))) {
 					ptr++;
 				}
 
