@@ -77,8 +77,6 @@ static struct nsgtk_resource_s ui_resource[] = {
 	RES_ENTRY("netsurf"),
 	RES_ENTRY("tabcontents"),
 	RES_ENTRY("password"),
-	RES_ENTRY("login"),
-	RES_ENTRY("ssl"),
 	RES_ENTRY("toolbar"),
 	RES_ENTRY("downloads"),
 	RES_ENTRY("globalhistory"),
@@ -88,6 +86,7 @@ static struct nsgtk_resource_s ui_resource[] = {
 	RES_ENTRY("cookies"),
 	RES_ENTRY("viewdata"),
 	RES_ENTRY("warning"),
+	RES_ENTRY("pageinfo"),
 	{ NULL, 0, NSGTK_RESOURCE_FILE, NULL },
 };
 
@@ -96,7 +95,18 @@ static struct nsgtk_resource_s pixbuf_resource[] = {
 	RES_ENTRY("favicon.png"),
 	RES_ENTRY("netsurf.xpm"),
 	RES_ENTRY("menu_cursor.png"),
-	RES_ENTRY("arrow_down_8x32.png"),
+	RES_ENTRY("icons/local-history.png"),
+	RES_ENTRY("icons/show-cookie.png"),
+	RES_ENTRY("icons/24x24/actions/page-info-insecure.png"),
+	RES_ENTRY("icons/24x24/actions/page-info-internal.png"),
+	RES_ENTRY("icons/24x24/actions/page-info-local.png"),
+	RES_ENTRY("icons/24x24/actions/page-info-secure.png"),
+	RES_ENTRY("icons/24x24/actions/page-info-warning.png"),
+	RES_ENTRY("icons/48x48/actions/page-info-insecure.png"),
+	RES_ENTRY("icons/48x48/actions/page-info-internal.png"),
+	RES_ENTRY("icons/48x48/actions/page-info-local.png"),
+	RES_ENTRY("icons/48x48/actions/page-info-secure.png"),
+	RES_ENTRY("icons/48x48/actions/page-info-warning.png"),
 	RES_ENTRY("throbber/throbber0.png"),
 	RES_ENTRY("throbber/throbber1.png"),
 	RES_ENTRY("throbber/throbber2.png"),
@@ -114,7 +124,6 @@ static struct nsgtk_resource_s direct_resource[] = {
 	RES_ENTRY("welcome.html"),
 	RES_ENTRY("credits.html"),
 	RES_ENTRY("licence.html"),
-	RES_ENTRY("maps.html"),
 	RES_ENTRY("default.css"),
 	RES_ENTRY("adblock.css"),
 	RES_ENTRY("internal.css"),
