@@ -371,7 +371,7 @@ window_init(const char *title,
 
 	res = nsgtk_builder_new_from_resname("viewdata", &newctx->builder);
 	if (res != NSERROR_OK) {
-		LOG("Viewdata UI builder init failed");
+		NSLOG(netsurf, INFO, "Viewdata UI builder init failed");
 		free(newctx);
 		return res;
 	}
@@ -381,7 +381,7 @@ window_init(const char *title,
 	window = GTK_WINDOW(gtk_builder_get_object(newctx->builder,
 						   "ViewDataWindow"));
 	if (window == NULL) {
-		LOG("Unable to find window in builder ");
+		NSLOG(netsurf, INFO, "Unable to find window in builder ");
 
 		/* free the builder */
 		g_object_unref(G_OBJECT(newctx->builder));
@@ -616,7 +616,7 @@ static char** xdg_data_strvec(void)
 			 xdg_data_home, xdg_data_dirs);
 	}
 
-	LOG("%s", xdg_data_path);
+	NSLOG(netsurf, INFO, "%s", xdg_data_path);
 
 	svec = filepath_path_to_strvec(xdg_data_path);
 	free(xdg_data_path);
@@ -651,7 +651,7 @@ static char *xdg_get_default_app(const char *path, const char *mimetype)
 	fname = malloc(fname_len);
 	snprintf(fname, fname_len, "%s/applications/defaults.list", path);
 
-	LOG("Checking %s", fname);
+	NSLOG(netsurf, INFO, "Checking %s", fname);
 
 	fp = fopen(fname, "r");
 	free(fname);
@@ -674,8 +674,11 @@ static char *xdg_get_default_app(const char *path, const char *mimetype)
 
 			ret = strdup(line + mimetype_len + 1);
 
-			LOG("Found line match for %s length %zu\n", mimetype, rd);
-			LOG("Result %s", ret);
+			NSLOG(netsurf, INFO,
+			      "Found line match for %s length %zu\n",
+			      mimetype,
+			      rd);
+			NSLOG(netsurf, INFO, "Result %s", ret);
 
 			break;
 		}
@@ -714,7 +717,7 @@ static char *xdg_get_exec_cmd(const char *path, const char *desktop)
 	fname = malloc(fname_len);
 	snprintf(fname, fname_len, "%s/applications/%s", path, desktop);
 
-	LOG("Checking %s", fname);
+	NSLOG(netsurf, INFO, "Checking %s", fname);
 
 	fp = fopen(fname, "r");
 	free(fname);
@@ -735,8 +738,8 @@ static char *xdg_get_exec_cmd(const char *path, const char *desktop)
 
 			ret = strdup(line + SLEN("Exec="));
 
-			LOG("Found Exec length %zu", rd);
-			LOG("Result %s", ret);
+			NSLOG(netsurf, INFO, "Found Exec length %zu", rd);
+			NSLOG(netsurf, INFO, "Result %s", ret);
 
 			break;
 		}
@@ -801,7 +804,7 @@ static char **build_exec_argv(const char *fname, const char *exec_cmd)
 
 		argv[aidx] = exec_arg(start, cur - start, fname);
 		if (argv[aidx] != NULL) {
-			LOG("adding \"%s\"", argv[aidx]);
+			NSLOG(netsurf, INFO, "adding \"%s\"", argv[aidx]);
 			aidx++;
 		}
 	}
