@@ -29,7 +29,7 @@
 
 typedef struct cmdhandler {
 	struct cmdhandler *r_next, *r_prev;
-	const char *cmd;
+	char *cmd;
 	handle_command_fn fn;
 } monkey_cmdhandler_t;
 
@@ -49,6 +49,17 @@ monkey_register_handler(const char *cmd, handle_command_fn fn)
 	return NSERROR_OK;
 }
 
+void
+monkey_free_handlers(void)
+{
+	while (handler_ring != NULL) {
+		monkey_cmdhandler_t *handler = handler_ring;
+		RING_REMOVE(handler_ring, handler);
+		free(handler->cmd);
+		free(handler);
+	}
+}
+
 void
 monkey_process_command(void)
 {
