@@ -45,6 +45,7 @@
 #include "riscos/window.h"
 #include "riscos/toolbar.h"
 #include "riscos/mouse.h"
+#include "riscos/wimputils.h"
 #include "riscos/corewindow.h"
 
 #ifndef wimp_KEY_END
@@ -143,10 +144,24 @@ static void ro_cw_scroll(wimp_scroll *scroll)
 	int page_y;
 	struct ro_corewindow *ro_cw;
 	wimp_open open;
+	wimp_window_state state;
 
 	ro_cw = (struct ro_corewindow *)ro_gui_wimp_event_get_user_data(scroll->w);
 	NSLOG(netsurf, INFO, "RO corewindow context %p", ro_cw);
 
+	state.w = ro_cw->wh;
+	error = xwimp_get_window_state(&state);
+	if (error) {
+		NSLOG(netsurf, INFO, "xwimp_get_window_state: 0x%x: %s",
+		      error->errnum, error->errmess);
+		return;
+	}
+
+	/* Don't try to update window if it's closed */
+	if (!(state.flags & wimp_WINDOW_OPEN)) {
+		return;
+	}
+
 	page_x = scroll->visible.x1 - scroll->visible.x0 - 32;
 	page_y = scroll->visible.y1 - scroll->visible.y0 - 32;
 
@@ -791,7 +806,7 @@ ro_cw_invalidate(struct core_window *cw, const struct rect *r)
 /**
  * Callback from the core to update the content area size.
  */
-static void
+static nserror
 ro_cw_update_size(struct core_window *cw, int width, int height)
 {
 	struct ro_corewindow *ro_cw = (struct ro_corewindow *)cw;
@@ -810,7 +825,7 @@ ro_cw_update_size(struct core_window *cw, int width, int height)
 	if (error) {
 		NSLOG(netsurf, INFO, "xwimp_get_window_state: 0x%x: %s",
 		      error->errnum, error->errmess);
-		return;
+		return NSERROR_INVALID;
 	}
 
 	/* only update the window if it is open */
@@ -823,16 +838,63 @@ ro_cw_update_size(struct core_window *cw, int width, int height)
 
 		update_scrollbars(ro_cw, &open);
 	}
+	return NSERROR_OK;
 }
 
 
 /**
  * Callback from the core to scroll the visible content.
  */
-static void
-ro_cw_scroll_visible(struct core_window *cw, const struct rect *r)
+static nserror
+ro_cw_get_scroll(const struct core_window *cw, int *x, int *y)
 {
-	//struct ro_corewindow *ro_cw = (struct ro_corewindow *)cw;
+	struct ro_corewindow *ro_cw = (struct ro_corewindow *)cw;
+	wimp_window_state state = {
+		.w = ro_cw->wh,
+	};
+	os_error *error;
+
+	error = xwimp_get_window_state(&state);
+	if (error) {
+		NSLOG(netsurf, ERROR, "xwimp_get_window_state: 0x%x: %s",
+				error->errnum, error->errmess);
+		return NSERROR_INVALID;
+	}
+
+	*x =  state.xscroll / 2;
+	*y = -state.yscroll / 2;
+	return NSERROR_OK;
+}
+
+
+/**
+ * Callback from the core to scroll the visible content.
+ */
+static nserror
+ro_cw_set_scroll(struct core_window *cw, int x, int y)
+{
+	struct ro_corewindow *ro_cw = (struct ro_corewindow *)cw;
+	wimp_window_state state = {
+		.w = ro_cw->wh,
+	};
+	os_error *error;
+
+	error = xwimp_get_window_state(&state);
+	if (error) {
+		NSLOG(netsurf, ERROR, "xwimp_get_window_state: 0x%x: %s",
+				error->errnum, error->errmess);
+		return NSERROR_INVALID;
+	}
+
+	state.xscroll =  x * 2;
+	state.yscroll = -y * 2;
+
+	/* only update the window if it is open */
+	if (state.flags & wimp_WINDOW_OPEN) {
+		update_scrollbars(ro_cw, PTR_WIMP_OPEN(&state));
+	}
+
+	return NSERROR_OK;
 }
 
 
@@ -843,8 +905,9 @@ ro_cw_scroll_visible(struct core_window *cw, const struct rect *r)
  * \param[out] width to be set to viewport width in px
  * \param[out] height to be set to viewport height in px
  */
-static void
-ro_cw_get_window_dimensions(struct core_window *cw, int *width, int *height)
+static nserror
+ro_cw_get_window_dimensions(const struct core_window *cw,
+		int *width, int *height)
 {
 	struct ro_corewindow *ro_cw = (struct ro_corewindow *)cw;
 	os_error *error;
@@ -855,29 +918,42 @@ ro_cw_get_window_dimensions(struct core_window *cw, int *width, int *height)
 	if (error) {
 		NSLOG(netsurf, INFO, "xwimp_get_window_state: 0x%x: %s",
 		      error->errnum, error->errmess);
-		return;
+		return NSERROR_INVALID;
 	}
 
 	*width = (state.visible.x1 - state.visible.x0) / 2;
 	*height = (state.visible.y1 - state.visible.y0) / 2;
+
+	/* Account for toolbar height, if present */
+	if (ro_cw->toolbar != NULL) {
+		*height -= ro_toolbar_full_height(ro_cw->toolbar) / 2;
+	}
+	if (*height < 0) {
+		*height = 0;
+	}
+
+	return NSERROR_OK;
 }
 
 
 /**
  * Callback from the core to update the drag status.
  */
-static void
+static nserror
 ro_cw_drag_status(struct core_window *cw, core_window_drag_status ds)
 {
 	struct ro_corewindow *ro_cw = (struct ro_corewindow *)cw;
 	ro_cw->drag_status = ds;
+
+	return NSERROR_OK;
 }
 
 
 struct core_window_callback_table ro_cw_cb_table = {
 	.invalidate = ro_cw_invalidate,
 	.update_size = ro_cw_update_size,
-	.scroll_visible = ro_cw_scroll_visible,
+	.set_scroll = ro_cw_set_scroll,
+	.get_scroll = ro_cw_get_scroll,
 	.get_window_dimensions = ro_cw_get_window_dimensions,
 	.drag_status = ro_cw_drag_status
 };
