@@ -17,21 +17,41 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** \file
- * Global history (interface).
+/**
+ * \file
+ * RISc OS global history interface.
  */
 
-#ifndef _NETSURF_RISCOS_GLOBALHISTORY_H_
-#define _NETSURF_RISCOS_GLOBALHISTORY_H_
+#ifndef RISCOS_GLOBALHISTORY_H
+#define RISCOS_GLOBALHISTORY_H
 
-#include "riscos/menus.h"
+/**
+ * initialise the global history window template ready for subsequent use.
+ */
+void ro_gui_global_history_initialise(void);
 
-void ro_gui_global_history_preinitialise(void);
-void ro_gui_global_history_postinitialise(void);
-void ro_gui_global_history_destroy(void);
-void ro_gui_global_history_open(void);
-void ro_gui_global_history_save(void);
+/**
+ * make the global history window visible.
+ *
+ * \return NSERROR_OK on success else appropriate error code on faliure.
+ */
+nserror ro_gui_global_history_present(void);
+
+/**
+ * Free any resources allocated for the global history window.
+ *
+ * \return NSERROR_OK on success else appropriate error code on faliure.
+ */
+nserror ro_gui_global_history_finalise(void);
+
+/**
+ * check if window handle is for the global history window
+ */
 bool ro_gui_global_history_check_window(wimp_w window);
+
+/**
+ * check if menu handle is for the global history menu
+ */
 bool ro_gui_global_history_check_menu(wimp_menu *menu);
 
 #endif
