@@ -73,10 +73,10 @@
 #include "riscos/query.h"
 #include "riscos/window.h"
 #include "riscos/iconbar.h"
-#include "riscos/sslcert.h"
 #include "riscos/local_history.h"
 #include "riscos/global_history.h"
 #include "riscos/cookies.h"
+#include "riscos/pageinfo.h"
 #include "riscos/wimp_event.h"
 #include "riscos/uri.h"
 #include "riscos/url_protocol.h"
@@ -1593,6 +1593,7 @@ static void gui_quit(void)
 	ro_gui_window_quit();
 	ro_gui_local_history_finalise();
 	ro_gui_global_history_finalise();
+	ro_gui_pageinfo_finalise();
 	ro_gui_hotlist_finalise();
 	ro_gui_cookies_finalise();
 	ro_gui_saveas_quit();
@@ -2427,12 +2428,10 @@ static struct gui_fetch_table riscos_fetch_table = {
 
 static struct gui_misc_table riscos_misc_table = {
 	.schedule = riscos_schedule,
-	.warning = ro_warn_user,
 
 	.quit = gui_quit,
 	.launch_url = gui_launch_url,
-	.cert_verify = gui_cert_verify,
-	.login = gui_401login_open,
+	.present_cookies = ro_gui_cookies_present,
 };
 
 
