@@ -21,7 +21,7 @@
  * This is only used if nothing claims Service_International,8
  */
 
-struct gui_utf8_table *riscos_utf8_table;
+extern struct gui_utf8_table *riscos_utf8_table;
 
 nserror utf8_to_local_encoding(const char *string, size_t len, char **result);
 nserror utf8_from_local_encoding(const char *string, size_t len, char **result);
