@@ -30,7 +30,12 @@
 #define IDR_TOOLBAR_BITMAP_HOT 104
 #define IDR_NETSURF_BANNER 105
 #define IDR_HOME_BITMAP 106
-
+#define IDC_PAGEINFO 107
+#define IDB_PAGEINFO_INSECURE 108
+#define IDB_PAGEINFO_SECURE 109
+#define IDB_PAGEINFO_INTERNAL 110
+#define IDB_PAGEINFO_WARNING 111
+#define IDB_PAGEINFO_LOCAL 112
 
 #define IDD_ABOUT 1000
 #define IDC_IMG1 1001
@@ -83,13 +88,6 @@
 #define IDC_PREFS_FANTASY 1219
 #define IDC_PREFS_FONTDEF 1220
 #define IDC_PREFS_NOANIMATION 1227
-#define IDC_PREFS_ANIMATIONDELAY 1228
-#define IDC_PREFS_ANIMATIONDELAY_SPIN 1229
-
-#define IDD_SSLCERT 1600
-#define IDC_SSLCERT_IMG1 1601
-#define IDC_SSLCERT_BTN_ACCEPT 1602
-#define IDC_SSLCERT_BTN_REJECT 1603
 
 #define IDD_LOGIN 1700
 #define IDC_LOGIN_USERNAME 1701
