@@ -16,19 +16,26 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _NETSURF_WINDOWS_WINDBG_H_
-#define _NETSURF_WINDOWS_WINDBG_H_
+#ifndef NETSURF_WINDOWS_WINDBG_H
+#define NETSURF_WINDOWS_WINDBG_H
 
 #include "utils/log.h"
 
 const char *msg_num_to_name(int msg);
 void win_perror(const char *lpszFunction);
 
-#define LOG_WIN_MSG(h, m, w, l)				\
-	if (((m) != WM_SETCURSOR) &&			\
-	    ((m) != WM_MOUSEMOVE) &&			\
-	    ((m) != WM_NCHITTEST) &&			\
-	    ((m) != WM_ENTERIDLE))			\
-		LOG("%s, hwnd %p, w 0x%x, l 0x%Ix", msg_num_to_name(m), h, w, l);
+/**
+ * log windows message
+ *
+ * log a windows message with symbols converted to human redable
+ */
+#define LOG_WIN_MSG(h, m, w, l)						\
+	if (((m) != WM_SETCURSOR) &&					\
+	    ((m) != WM_MOUSEMOVE) &&					\
+	    ((m) != WM_NCHITTEST) &&					\
+	    ((m) != WM_ENTERIDLE))					\
+		NSLOG(netsurf, DEBUG,					\
+		      "%s, hwnd %p, w 0x%x, l 0x%Ix",			\
+		      msg_num_to_name(m), h, w, l)
 
 #endif
