@@ -42,13 +42,15 @@ void nslog_log(const char *file, const char *func, int ln, const char *format, .
 {
 	va_list ap;
 
-	fprintf(stderr, "%s:%i %s: ", file, ln, func);
+	if (verbose_log) {
+		fprintf(stderr, "%s:%i %s: ", file, ln, func);
 
-	va_start(ap, format);
+		va_start(ap, format);
 
-	vfprintf(stderr, format, ap);
+		vfprintf(stderr, format, ap);
 
-	va_end(ap);
+		va_end(ap);
 
-	fputc('\n', stderr);
+		fputc('\n', stderr);
+	}
 }
