@@ -36,6 +36,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
+#include <inttypes.h>
 
 #include "utils/ascii.h"
 #include "utils/corestrings.h"
@@ -235,6 +236,15 @@ lwc_string *nsurl_get_component(const nsurl *url, nsurl_component part)
 }
 
 
+/* exported interface, documented in nsurl.h */
+enum nsurl_scheme_type nsurl_get_scheme_type(const nsurl *url)
+{
+	assert(url != NULL);
+
+	return url->components.scheme_type;
+}
+
+
 /* exported interface, documented in nsurl.h */
 bool nsurl_has_component(const nsurl *url, nsurl_component part)
 {
@@ -915,7 +925,7 @@ nserror nsurl_parent(const nsurl *url, nsurl **new_url)
 	} else if (old_path_len == new_path_len) {
 		lwc_path = lwc_string_ref(url->components.path);
 	} else {
-		if (lwc_intern_string(path, old_path_len - new_path_len,
+		if (lwc_intern_string(path, new_path_len,
 				&lwc_path) != lwc_error_ok) {
 			free(*new_url);
 			return NSERROR_NOMEM;
@@ -955,3 +965,36 @@ nserror nsurl_parent(const nsurl *url, nsurl **new_url)
 
 	return NSERROR_OK;
 }
+
+/* exported interface, documented in nsurl.h */
+void nsurl_dump(const nsurl *url)
+{
+	fprintf(stderr, "nsurl components for %p "
+			"(refs: %i hash: %"PRIx32"):\n",
+			url, url->count, url->hash);
+
+	if (url->components.scheme)
+		fprintf(stderr, "  Scheme: %s\n",
+				lwc_string_data(url->components.scheme));
+	if (url->components.username)
+		fprintf(stderr, "Username: %s\n",
+				lwc_string_data(url->components.username));
+	if (url->components.password)
+		fprintf(stderr, "Password: %s\n",
+				lwc_string_data(url->components.password));
+	if (url->components.host)
+		fprintf(stderr, "    Host: %s\n",
+				lwc_string_data(url->components.host));
+	if (url->components.port)
+		fprintf(stderr, "    Port: %s\n",
+				lwc_string_data(url->components.port));
+	if (url->components.path)
+		fprintf(stderr, "    Path: %s\n",
+				lwc_string_data(url->components.path));
+	if (url->components.query)
+		fprintf(stderr, "   Query: %s\n",
+				lwc_string_data(url->components.query));
+	if (url->components.fragment)
+		fprintf(stderr, "Fragment: %s\n",
+				lwc_string_data(url->components.fragment));
+}
